C*************************************************************************
C     FORTRAN77 code
C     File: gva.for
C     Coded by: Antoni Vidiella-i-Anguera
C
C     A SIMULATION PROGRAM FOR TESTING A IDENTIFICATION STRATEGY (GV)
C     OF SEASONAL AUTOREGRESSIVE TIME SERIES MODELS
C     SIMULATED MODEL (1-0.5B)(1-0.5B^4)Y(t)=a(t)
C     N      = SAMPLE SIZE OF THE SERIES
C     ID1MAX = MAX. DEGREE OF REGULAR DIFFERENCING 
C     ID2MAX = MAX. DEGREE OF SEASONAL DIFFERENCING
C     IP1MAX = MAX. ORDER OF NON.SEASONAL AR 
C     IS     = PERIOD OF SEASONALITY
C     METHOD OF ESTIMATION: LEAST SQUARES & SCORE
C************************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER THMAX,SEED
C
C     *** PARAMETER VARIABLES ***
C
      PARAMETER (LMAX=50,NMAX=800,THMAX=1)
C
C     *** REAL VECTORS AND MATRICES ***
C      
      DIMENSION Y(NMAX),AC1(LMAX),ZTHD(LMAX,3)
      DIMENSION AX(NMAX,LMAX,THMAX+1),THD(THMAX),KKARAY(THMAX+1)
      DIMENSION NOBS(THMAX+1),SAX(NMAX,LMAX),A(LMAX,LMAX)
      DIMENSION SAG(NMAX,LMAX),AB0(LMAX),AB1(LMAX),DYDA(LMAX)
      DIMENSION XI(LMAX),Y3(NMAX+200)
      DIMENSION SDM(LMAX),YKL(1,1),YI(1,LMAX),d2yda(lmax,lmax)
      DIMENSION B3(LMAX),RI(1000),B11(LMAX,1),SAXI(NMAX,LMAX)
      DIMENSION B1(LMAX),B0(LMAX),C5(LMAX),A0(LMAX),D0(LMAX)
      DIMENSION A2(LMAX),C2(LMAX),Q(LMAX,LMAX),B(LMAX,LMAX)
      DIMENSION B2(LMAX,LMAX),ELP(LMAX,LMAX),B9(LMAX,1),Q1(LMAX,LMAX)
C
C     *** EXTERNAL SUBROUTINES ***
C
      EXTERNAL ZEROMAT,ZEROVEC,GNORM,EQMAT,PRELIM,MATINV
      EXTERNAL SORT,IZEROVEC,ZEROMAT3,GAP
      EXTERNAL ORDER,HUSHLD,EQVEC,MULTEST,MRQCOF
      EXTERNAL ARMFIT,VARCOV,SINV,MULTMAT

C
C     *** SIMULATION DATA ***
      NSIM=200
      N= 211
      IS = 4
      IIP2 = 2
      ID1MAX=2
      ID2MAX=2
      CRITIC1=5.99147
      CRITIC2=3.84146
      CRITIC3=CRITIC1+CRITIC2
c
C     *** OPEN RESULTS FILE ***
C
      OPEN (3,FILE='gvsim.res',STATUS='UNKNOWN')
C
C     *** LIMITING NUMBER OF ITERATIONS
C
	I0=50
C**********************************************************************
C     SIMULATION NON MULTPLICATIVE STATIONARY AR PROCESS PARAMETERS
C
C**********************************************************************
C
C     *** INTRODUCTION OF PARAMETERS ***
C
      NPAM=5
      B3(1)=0.5
      B3(2)=0.0
      B3(3)=0.0
      B3(4)=0.5
      B3(5)=-0.25
      SEED=667777
      THD(1)=0.0      
      IGAP=0
      N3=N
      IWRONG=0
      IORDER=0
      IMULT=0
      ITOT=0
      IWRONG3=0
      IWRONG2=0
      ifalse=0
      IKL=0
C     *** SIMULATION START ***
      DO 150 I=1,NSIM
      write(6,*) i
C
C     *** AR(IP1,IP2,S) MODEL GENERATION *** 
C
         N=N3
         CALL ZEROVEC(Y,NMAX,N)
         CALL ZEROVEC(Y3,NMAX+200,N)
         CALL ZEROVEC(RI,1000,1000)
         CALL GNORM(RI,1000,SEED)
         DO 12 IT0=1,NPAM
            B11(IT0,1)=B3(IT0)
 12         YI(1,IT0)=RI(100+IT0)
            DO 15 IT3 = 1, N+200
               CALL MULTMAT(YI,B11,YKL,1,NMAX,1,NPAM,1)
               Y3(IT3)=YKL(1,1)+RI(250+IT3)              
               DO 17 IT0=0,NPAM-2
                  IK=NPAM-IT0
 17               YI(1,IK)=YI(1,IK-1)
               YI(1,1)=Y3(IT3)
 15      CONTINUE
         DO 16 I1=1,N
 16         Y(I1)=Y3(I1+200)
         write(3,*) seed

C**********************************************************************
C     SEARCHING PROCEDURE OF THE BEST ORDERS
C     
C**********************************************************************
         IER=0
         IES=0
         THD(1)=0.0
         KKARAY(1)=(IS-1)+IIP2*IS
         ITF=KKARAY(1)
         CALL ZEROMAT3(AX,NMAX,LMAX,N,ITF,THMAX+1)
         CALL ZEROMAT(ZTHD,LMAX,ITF,3)
         CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,0,0,NOBS,IER,IES)
         N=NOBS(1)
         IC=0
         DO 5126 IP2=0,IIP2
            DO 5128 IP1=0,IS-1
               IC=IC+1
               KR=IP1+IP2*IS
               KRL=IP1+1
               K1=KR+2
               K2=KR+1
               IL=IP1+IP2*KRL
               CALL ZEROMAT(SAG,NMAX,N,K1)
               DO 5129 I1=1,N
                  DO 5131 I8=1,IP2+1
		         DO 5131 I2=1,KRL
                         SAG(I1,(I8-1)*KRL+I2)=AX(I1,IS*(I8-1)+I2,1)
 5131             CONTINUE        
                  SAG(I1,(I8-1)*KRL+1)=AX(I1,ITF+2,1)
 5129          CONTINUE                         
               CALL HUSHLD(SAG,N,IL+2,NMAX)
               CALL ARMFIT(SAG,NMAX,AC1,IL+1,N,VA,IMIN,SIC)
               ZTHD(IC,1)=REAL(IP1)
               ZTHD(IC,2)=REAL(IP2)
               ZTHD(IC,3)=SIC
 5128       CONTINUE
 5126    CONTINUE
         IOP=1
         ZTHDM=ZTHD(1,3)
         DO 5106 I1=2,IC
            IF (ZTHD(I1,3).LT.ZTHDM) THEN 
               ZTHDM=ZTHD(I1,3)
               IOP=I1
	    END IF         
 5106    CONTINUE
         HQ=0
         write(3,*) INT(ZTHD(IOP,1)),INT(ZTHD(IOP,2))
         IF((INT(ZTHD(IOP,1)).EQ.1).AND.(INT(ZTHD(IOP,2)).EQ.1)) THEN
            HQ=1
            IORDER=IORDER+1
            GOTO 300
         ELSE
            HQ=2
            IWRONG=IWRONG+1
            GOTO 150
         END IF 
 300     CONTINUE
C
C *** GAPS ARE TESTED FOR THOSE MODELS CORRECTLY ESTIMATED ***
C           
         IP1=1
         IP2=1
         I9=0
         K1=1+IP1+IS*IP2
         IL=IP1+IP2*(IP1+1)
 1900    IF (I9.GT.0) GOTO 2300
C
C     COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE GAPS MODEL
C
         CALL ZEROMAT(SAX,NMAX,N,K1)
         CALL ZEROMAT(SAXI,NMAX,N,K1)
         DO 26 I1=1,N
	      DO 28 I2=1,K1
 28            SAX(I1,I2)=AX(I1,I2,1)
 26            SAX(I1,K1+1)=AX(I1,ITF+2,1)
         CALL EQMAT(SAX,SAXI,NMAX,N,K1+1)
         CALL HUSHLD(SAX,N,K1+1,NMAX)
         CALL ZEROVEC(AC1,LMAX,K1)
         CALL ARMFIT(SAX,NMAX,AC1,K1,N,VA,IMIN,SIC2)
         CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
         CALL ZEROVEC(B1,LMAX,K1)
         DO 27 I1=1,K1
 27         B1(I1)=AC1(I1)
         S2=VA
 2300    CONTINUE
         CALL ZEROVEC(B0,LMAX,K1)
         CALL EQVEC(B1,B0,LMAX,K1)
         CALL ZEROMAT(A,LMAX,K1,K1)
         CALL ZEROVEC(A2,LMAX,K1)
         CALL ZEROMAT(D2YDA,LMAX,K1,K1)
         CALL MRQCOF(SAXI,NMAX,S2,N,B1,K1,A,A2,LMAX,D2YDA,DYDA,XI,i)
         CALL ZEROVEC(C5,LMAX,K1)
         if (dabs(a2(k1+1)).gt.1.d-10) then
            write(3,*) (a2(ik),ik=1,k1+1)
            if (hq.eq.1) iorder=iorder-1
            if (hq.eq.2) iwrong=iwrong-1
            ifalse=ifalse+1
            goto 170
          end if
         CALL GAP(A2,C2,C5,A0,LMAX,A,Q,B,B2,
     1            ELP,B9,B11,LMAX,IP1,IP2,IS)
         D9=DABS(A0(1))
         DO 74 I1=1,K1+1
 74         IF(D9.LT.DABS(A0(I1))) D9=DABS(A0(I1))
         CALL ZEROVEC(B1,LMAX,K1+1)
         CALL ZEROVEC(AB0,LMAX,K1+1)     
         CALL ORDER(B0,AB0,LMAX,IP1,IP2,IS,0)
c         write(3,*) (a0(ik),ik=1,k1)
         DO 76 I1=1,K1            
 76         B1(I1)=AB0(I1)+A0(I1)
         S2=S2+A0(K1+1)
         if (i9.gt.0) write(6,*) i,i9
         IF (D9.LE.(1.D-3).OR.I9.EQ.I0) GOTO 2800
         CALL ZEROVEC(AB1,LMAX,K1)
         CALL ORDER(B1,AB1,LMAX,IP1,IP2,IS,1)
         CALL EQVEC(AB1,B1,LMAX,K1)
         I9=I9+1
         GOTO 1900
 2800    CONTINUE
         C3=0.0
         HY=0
         DO 90 I1=IL+2,K1
            DO 90 I2=IL+2,K1
 90            C3=C3+B1(I1)*B1(I2)*B(I1,I2)
         C3=N*C3
         IF (C3.LE.CRITIC1) THEN
            HY=1
            IGAP=IGAP+1
            GOTO 500
         ELSE
            HY=2
            IWRONG2=IWRONG2+1
            GOTO 150
         END IF
 500     CONTINUE
C
C   ***  TESTING OF MULTIPLICATIVE SEASONAL MODEL ***
         S2=0.0
         I5=IP1+IP2*(IP1+1)
         I9=0
C	  
C       COMPUTATION OF THE SCORE VECTOR AND INFORMATION MATRIX OF THE
C       SEASONAL MODEL 
C
 600     IF (I9.GT.0) GOTO 730
C
C       COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE SEASONAL MODEL
C               
         K2=IP1+IP2*(IP1+1)+1
         K1=K2+1
         CALL ZEROVEC(B1,LMAX,K2)          
         CALL ZEROMAT(SAX,NMAX,N,K1)
         CALL ZEROMAT(SAXI,NMAX,N,K1)
         CALL ZEROVEC(SDM,LMAX,K2)
         DO 610 I1=1,N
            DO 612 I2=1,IP1+1
 612           SAX(I1,I2)=AX(I1,I2,1)
            DO 614 I3=1,IP2
               DO 614 I4=1,IP1+1
 614              SAX(I1,1+IP1+I4+(I3-1)*(IP1+1))=AX(I1,I4+IS*I3,1)
 610           SAX(I1,IP1+IP2*(IP1+1)+2)=AX(I1,ITF+2,1)
         CALL EQMAT(SAX,SAXI,NMAX,N,K1)
         CALL HUSHLD(SAX,N,K1,NMAX)
         CALL ARMFIT(SAX,NMAX,AC1,K2,N,VA,IMIN,SIC2)
         CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
         DO 616 I1=1,K2
 616        B1(I1)=AC1(I1)
         S2=VA 
         CALL EQVEC(B1,B0,LMAX,K2)
C     *** INTRODUCTION PRELIMINAR PARAMETERS ***
         CALL ZEROVEC(C5,LMAX,I5+2)
         CALL PRELIM(B0,D0,C5,LMAX,B1,LMAX,IP1,IP2,IS)
         do 725 i1=ip1+ip2+2,i5+1
 725        b0(i1)=-b0(i1)
         CALL EQVEC(B0,B1,LMAX,I5+1)
 730     CONTINUE                 
         CALL ZEROVEC(A0,LMAX,K2)
         if (s2.lt.1.d-6.or.s2.gt.1.d+6) then
            if (hq.eq.1) iorder=iorder-1
            if (hq.eq.2) iwrong=iwrong-1
            if (hy.eq.1) igap=igap-1
            if (hy.eq.2) iwrong2=iwrong2-1
            ifalse=ifalse+1
            goto 170
         end if
         CALL MRQCOF(SAXI,NMAX,S2,N,B1,K2,A,A2,LMAX,d2yda,DYDA,XI,i)
c         write(3,*) (a2(ik),ik=1,k1)
c         do 73 ik=1,k1
c 73         write(3,*) (a(ik,ij),ij=1,k1)

C
C         MULTIPLICATIVE TESTING
C
         CALL MULTEST(A2,D0,C2,A0,A,Q,B,B2,ELP,B9,B11,LMAX,
     1                   IP1,IP2,IS,i9)
         D9=ABS(A0(1))
         DO 618 I6=2,I5+2
 618        IF (D9.LT.ABS(A0(I6))) D9=ABS(A0(I6))
         CALL ZEROVEC(B1,LMAX,I5+1)
         DO 620 I6=1,I5+1               
 620           B1(I6)=B0(I6)+A0(I6)
c 620     write(3,*) b1(i6),b0(i6),a0(i6),i,i9
         S2=S2+A0(I5+2)               
         IF (D9.LT.(1.D-3).OR.I9.EQ.I0) GOTO 780
         CALL EQVEC(B1,B0,LMAX,I5+1)
               DO 622 I6=1,IP1+1
 622              D0(I6)=B0(I6)
               DO 624 I1=1,IP2
                  DO 624 I2=1,IP1+1
 624                 D0(IS*I1+I2)=B0(IP1+I2+(I1-1)*(IP1+1)+1)
         I9=I9+1
         GOTO 600
 780     CONTINUE
         CALL ZEROMAT(Q,LMAX,IP1*IP2,IP1*IP2)
         CALL ZEROMAT(Q1,LMAX,IP1*IP2,IP1*IP2)
         DO 460 I1=1,IP1*IP2
            DO 460 I2=I1,IP1*IP2
               Q(I1,I2)=-B(I5+2+I1,I5+2+I2)
 460           Q(I2,I1)=Q(I1,I2)
         CALL ZEROMAT(ELP,LMAX,IP1*IP2,IP1*IP2)
         CALL MATINV(Q,Q1,ELP,LMAX,IP1*IP2,IFAIL)
         C4=0.0
         DO 470 I1=1,IP1*IP2
            DO 470 I2=1,IP1*IP2
 470           C4=C4+A0(I5+I1+2)*A0(I5+I2+2)*Q1(I1,I2)
         C4=N*C4
         IF (C4.LE.CRITIC2) IMULT=IMULT+1
         IF ((C3+C4).LE.CRITIC3) THEN
            ITOT=ITOT+1
         ELSE
            IWRONG3=IWRONG3+1
         END IF
 170  continue
 150  CONTINUE
      WRITE(3,901)
      WRITE(3,902) N
      WRITE(3,903)
      WRITE(3,902) IP1
      WRITE(3,904)
      WRITE(3,902) IIP2
      WRITE(3,905)
      WRITE(3,902) IS
      WRITE(3,906)
      WRITE(3,902) NSIM-IFALSE
      WRITE(3,911)
      WRITE(3,912) B3(1)
      WRITE(3,913) B3(2)
      WRITE(3,914) B3(3)
      WRITE(3,915) B3(4)
      WRITE(3,916) B3(5)
      WRITE(3,9000)
      WRITE(3,9001) IORDER
      WRITE(3,9002)
      WRITE(3,9001) IGAP
      WRITE(3,9003)
      WRITE(3,9001) IMULT
      WRITE(3,9004)
      WRITE(3,9001) ITOT
      WRITE(3,9005)
      WRITE(3,9001) IORDER-IGAP
      WRITE(3,9006)
      WRITE(3,9001) IGAP-IMULT
      WRITE(3,9007)
      WRITE(3,9001) IWRONG
      WRITE(3,9008)
      WRITE(3,9001) IGAP-ITOT
      write(3,9001) ifalse
 9000 FORMAT ('CORRECT ORDER (p=1,P=1)')
 9001 FORMAT (' ',I4)
 9002 FORMAT ('NOT REJECTED GAPS @ 5% (only those correct order)')
 9003 FORMAT ('NOT REJECTED MULTIPLICATIVE RESTRICTION @ 5% (iff gaps no
     1t rejected)')
 9004 FORMAT ('NOT REJECTED SEASONAL NULL HYPOTHESIS (Gaps+Mult)')
 9005 FORMAT ('REJECTED GAPS @ 5% (only those correct order) ')
 9006 FORMAT ('REJECTED MULTIPLICATIVE RESTRICTION @ 5% ')
 9007 FORMAT ('WRONG ORDER (p ne 1 or P ne 1)')
 9008 FORMAT ('REJECTED SEASONAL HYPOTHESIS @ 5%')
  901 FORMAT ('EFECTIVE SAMPLE SIZE:') 
  902 FORMAT ('',I5)
  903 FORMAT ('AUTOREGRESSIVE REGULAR ORDER:')
  904 FORMAT ('AUTOREGRESSIVE SEASONAL ORDER:')
  905 FORMAT ('SEASONAL PERIODICITY:')
  906 FORMAT ('NUMBER OF SIMULATIONS:')
  911 FORMAT ('SIMULATED MODEL:')
  912 FORMAT ('PHI(1): ',F10.3)
  913 FORMAT ('PHI(2): ',F10.3)
  914 FORMAT ('PHI(3): ',F10.3)
  915 FORMAT ('PHI(4): ',F10.3)
  916 FORMAT ('PHI(5): ',F10.3)
  917 FORMAT ('PHI(6): ',F10.3)
  918 FORMAT ('PHI(7): ',F10.3)
  919 FORMAT ('PHI(8): ',F10.3)

      END

      SUBROUTINE ZEROMAT(AMAT,ID1MAX,L1,L2)
C**************************************************************
C     PURPOSE: CREATE A MATRIX OF ZEROS
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: AMAT, ZEROED MATRIX DIMENSION L1,L2
C**************************************************************      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*)
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    AMAT(I,J)=0.0
  99     CONTINUE
  98  CONTINUE
      RETURN
      END

      SUBROUTINE ZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PURPOSE: CREATE A VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VEC(ID1MAX)
      DO 98 I=1,L1     
         VEC(I)=0.0
  98  CONTINUE
      RETURN
      END   

      SUBROUTINE EQVEC(VECA,VECB,ID1MAX,L1)
C******************************************************************
C     PURPOSE: EQUALIZATION OF VECTORS
C     INPUT: VECA, VECTOR DIMENSION L1
C     OUTPUT: VECB, VECTOR EQUAL VECA DIMENSON L1
C******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VECA(ID1MAX),VECB(ID1MAX)
C
      DO 99 I=1,L1
         VECB(I)=VECA(I)
   99 CONTINUE
      RETURN
      END

      SUBROUTINE SORT(Y,N,AX,IAX1,IAX2,KKARAY,THD,NTHD,ID,NOBS,IER,IES)
C***********************************************************************
C     PURPOSE: SORT THE DATA FOR WHICH AN APPROPIATE SAR-MODEL APPLIES
C     INPUT: Y, DATA
C            N, ITS DIMENSION
C            THD, ARRAY CONTAINING THE THRESHOLDS
C            NTHD+1, NUMBER OF PIECES
C            ID, DELAY PARAMETER
C     OUTPUT:AX, ARRAY CONTAINING THE SORTED DATA
C            IER, ERROR CODE 0 OK
C                            1 SOME PIECES DO NOT HAVE ENOUGH DATA TO
C                                    ESTIMATE THE AR-MODEL
C            IES, START POSITION OF SORTING
C
C            NOBS(I), NUMBER OF DATA IN THE I-TH PIECE
C ***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION NOBS(*),Y(*),THD(*)
      DIMENSION AX(IAX1,IAX2,1),KKARAY(*)
C     *** NUMBER OF PIECES ***
      JEND=NTHD+1
C     *** CLEANING NOBS  ***
      DO 10 I=1,JEND
 10      NOBS(I)=0

      
C     ***  CHOSE APPROPIATE STARTING POSITION  ***
      IES=ID
      DO 20 I1=1,JEND
	 KK=KKARAY(I1)
 20      IES=MAX0(IES,KK)
      IES=IES+1
C     *** SORT X INTO AX'IS ***
      DO 30 I=IES,N
	 ICP=1
C     *** ASSIGN TO FIRST PIECE FOR NTHD=0 ***
	 IF (NTHD.EQ.0) GOTO 50
C     *** FIRST ASSIGN TO LAST PIECE ***
	 ICP=NTHD+1
         I1=I-ID      
C     *** SEE Y(IT) BELONGS TO WHICH PIECE
	 DO 40 K=1,NTHD
	    IF (Y(I1).GT.THD(K)) GOTO 40
	    ICP = K
C     *** AH, IT'IS FOUND ***
	    GOTO 50
  40     CONTINUE
  50     CONTINUE
C     *** INDICATE THE CURRENT POSITION X(I) OCCUPIES IN THE ICP PIECE
	 NOBS(ICP)=NOBS(ICP)+1
	 NOB=NOBS(ICP)
	 KEND=KKARAY(ICP)+1
C     *** ASSIGN AX(NOB,I)
	 DO 60 K=1,KEND
            AX(NOB,K,ICP)=Y(I-K+1)
  60     CONTINUE
C     *** REASSIGN AX TO AVOID CASE KK=0
	 AX(NOB,1,ICP)=1.0
	 KEND=KEND+1
	 AX(NOB,KEND,ICP)=Y(I)
  30  CONTINUE
C     *** CHECK IF ENOUGH DATA TO ESTIMATE THE MODEL ***
      IER=0
      DO 70 J=1,JEND
	 NN=NOBS(J)
	 IF (NN.GE.60) GOTO 70
	 IER=1
C     *** The JTH REGIME LACKS DATA, SO RETURN WITH IER=1 ***
	 RETURN
  70  CONTINUE
      RETURN
      END

C     
      SUBROUTINE ZEROMAT3(AMAT,ID1MAX,ID2MAX,L1,L2,L3)
C************************************************************
C
C     PURPOSE: CREATE A 3D MATRIX OF ZEROS   
C     INPUT: AMAT, MATRIX TO BE ZEROED
C            L1,L2,L3, DIMENSIONS
C     OUTPUT: AMAT, ZEROED MATRIX
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,ID2MAX,*)
      DO 97 M=1,L3
	 DO 98 J=1,L2
	    DO 99 I=1,L1
	       AMAT(I,J,M)=0.0
  99        CONTINUE
  98     CONTINUE
  97   CONTINUE
C
      RETURN
      END

      SUBROUTINE IZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PURPOSE: CREATE A INTEGER VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      INTEGER ID1MAX,L1,VEC
      DIMENSION VEC(*)
      DO 98 I=1,L1     
         VEC(I)=0
  98  CONTINUE
      RETURN
      END

      SUBROUTINE HUSHLD(AH1,N,K,IA11)
C*****************************************************************
C Purpose: HOUSHOLDER TRANSFORMATION
C Input : AH1 = array to be transformed;
C         N = number of columns;
C         K = number of rows;
C         IA11 = number of columns of AH1 in the calling program.
C Output: AH1 = transformed array.
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*),Z(5000)
      TOL=1.0E-30
      NK=MIN0(N,K)
      DO 100 II=1,NK
         H=0.0
         DO 10 I=II,N
            Z(I) = AH1(I,II)
   10       H=H+Z(I)*Z(I)
         IF(H .GT. TOL) GOTO 40
         G=0.0
         GOTO 100
   40    G=SQRT(H)
         F=AH1(II,II)
         IF(F.GE.0.0) G=-G
         Z(II)=F-G
         H=H-F*G
         IF(II .EQ. K) GOTO 100
         II1=II+1
         DO 90 J=II1,NK
         S=0.0
         DO 20 I=II,N
   20       S=S+Z(I)*AH1(I,J)
         S=S/H
         DO 30 I=II,N
   30       AH1(I,J)=AH1(I,J)-Z(I)*S
   90    CONTINUE
  100    AH1(II,II)=G
      RETURN
      END

      SUBROUTINE ARMFIT(AH1,IA11,AC1,K,N,VA,IMIN,AC3)
C*******************************************************************
C Purpose: given the lags for different pieces, compute the AIC
C          for the AR-model in the Ith piece.
C Input: AH1 = housholder transformed array (so it is upper triangular);
C        IA11 = number of rows of AH1 in the calling program;
C        K = number of columns;
C        N = number of rows (number of data);
C        ISEL = indicator for information criterion.
C Output: AR(IMIN-1) = the chosen AR-model;
C         IMIN = the number parameters of the AR-model;
C         AICM = its corresponding information criterion;
C         VA = mean sum of squared errors;
C         AC1 = contains the coefficients of the AR-model.
C Here, AC3 is SIC 
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*)
      DIMENSION AC1(*)
      K1=K+1
      K1=MIN0(N,K1)
      OSD=0.0E00
      FN=N
      OSD=OSD+AH1(K1,K1)*AH1(K1,K1)
      AC2=OSD/FN
      AC3=FN*LOG(AC2)+K*LOG(REAL(FN))
      VA=AC2
      IMIN=K1-1
      IF(IMIN .EQ. 1) GOTO 200
      DO 100 M=IMIN,IMIN
	 AC1(M)=AH1(M,K1)/AH1(M,M)
	 MM1=M-1
	 DO 110 II=1,MM1
	    I=M-II
	    SUM=DBLE(AH1(I,K1))
	    I1=I+1
	    DO 120 J=I1,M
  120       SUM=SUM-DBLE(AC1(J))*DBLE(AH1(I,J))
  110    AC1(I)=SUM/AH1(I,I)
  100 CONTINUE
      RETURN
  200 AC1(1)=AH1(1,K1)/AH1(1,1)
      RETURN
      END

      SUBROUTINE VARCOV(SAX,IAX,IMIN,VA,SDM)
C*****************************************************************
C PURPOSE: TO PROVIDE THE VARIANCE COVARIANCE MATRIX FOR THE
C          ESTIMATES.
C INPUT: SAX=UPPER TRIANGULAR FORM OF THE DESIGN MATRIX
C        IAX=NO. OF COLUMNS OF SAX IN THE CALLING PROGRAM
C        IMIN: SAX IS OF DIMENSION IMIN BY IMIN
C        VA=SUM OF SQUARED ERRORS
C ACTION: THE SD ESTIMATES AND THE VARIANCE COVARIANCE MATRIX IS PRINTED
C SUBROUTINES CALLED: SINV
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SAX(IAX,*),SDM(*)
      DIMENSION TA(5000)
      EXTERNAL SINV
      IC=0
C     TO COMPRESS THE UPPER TRIANGULAR MATRIX SAX INTO ONE
C     DIMENSIONAL VECTOR TA
      DO 2 I2=1,IMIN
         DO 2 I1=1,I2
            IC=IC+1
    2 TA(IC)=SAX(I1,I2)
C     CALCULATE VA*INVERSE OF THE PRODUCT OF SAX TRANSPOSE AND SAX
      CALL SINV(TA,IMIN,VA)   
C     RETIREVE THE DIAGONAL AND TAKE THE SQUARE ROOT; THEY ARE
C     ESTIMATES OF STANDARD ERROR
      I=1
      IP=1
 11   SDM(I)=SQRT(TA(IP))
      I=I+1
      IP=I+IP
      IF(I .GT. IMIN) GOTO 10
      GOTO 11
  10  RETURN
      END

      SUBROUTINE SINV(A,N,VA)
C********************************************************************
C PURPOSE: TO PROVIDE THE PRODUCT OF THE INVERSE A'*A AND VA
C INPUT: A THE UPPER TRIANGULAR MATRIX STACKED UP ROW-WISE
C        N=DIMENSION OF THE TRIANGULAR MATRIX,SO N*(N-1)=LENGTH OF A
C        VA=MEAN SUM OF SQUARE ERROR
C OUTPUT: A=THE DESIRED PRODUCT AS DESCRIBED IN THE PURPOSE
C********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(5000)
      IPIV=N*(N+1)/2
      IND=IPIV
      DO 6 I=1,N
      DIN=1.D0/DBLE(A(IPIV))
      A(IPIV)=DIN
      MIN=N
      KEND=I-1
      LANF=N-KEND
      IF(KEND) 5,5,2
    2 J=IND
      DO 4 K=1,KEND
      WORK=0.D0
      MIN=MIN-1
      LHOR=IPIV
      LVER=J
      DO 3 L=LANF,MIN
      LVER=LVER+1
      LHOR=LHOR+L
    3 WORK=WORK+DBLE(A(LVER)*A(LHOR))
      A(J)=-WORK*DIN
    4 J=J-MIN
    5 IPIV=IPIV-MIN
    6 IND=IND-1
      DO 8 I=1,N
      IPIV=IPIV+I
      J=IPIV
      DO 8 K=I,N
      WORK=0.D0
      LHOR=J
      DO 7 L=K,N
      LVER=LHOR+K-I
      WORK=WORK+DBLE(A(LHOR)*A(LVER))
    7 LHOR=LHOR+L
      A(J)=WORK*DBLE(VA)
    8 J=J+K
      RETURN
      END

      SUBROUTINE MULTMAT(AMAT,BMAT,CMAT,ID1MAX,ID2MAX,L1,L2,L3)
C**********************************************************************
C     PORPOSE: MATRIX MULTIPLICATION
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C            BMAT, MATRIX DIMENSION L2,L3
C     OUPUT: CMAT, MATRIX DIMENSION L1,L3
C**********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID2MAX,*),CMAT(ID1MAX,*)
      EXTERNAL ZEROMAT
      CALL ZEROMAT(CMAT,ID1MAX,L1,L3)
      DO 99 J=1,L3
	 DO 98 K=1,L2
	    IF (BMAT(K,J).EQ.0.0) GOTO 98
	    DO 97 I=1,L1
	       CMAT(I,J)=CMAT(I,J)+AMAT(I,K)*BMAT(K,J)
   97       CONTINUE
   98    CONTINUE
   99 CONTINUE
      RETURN
      END

       subroutine gnorm(r,nr,rs)
C       IMPLICIT REAL (A-H,O-Z)
       integer nr,rs
       real*8 r(nr)
       do  1 i=1,nr
99        continue         
          v=randu(rs)*2.-1.
          u=randu(rs)*2.-1.
          w=v*v+u*u
          if(w.gt.1)goto 99
          c=sqrt(-2.*alog(w)/w)
          R(i)=c*v
          r(i+1)=c*u
1      continue
       return
       end                      
	   	   
C**************************************************************************  
C  
      REAL FUNCTION RANDU(IX)  
C  
C UNIFORM RANDOM NUMBER GENERATOR.  
C REF: SCHRAGE, L. 1979 'A MORE PORTABLE FORTRAN RANDOM NUMBER GENERATOR'  
C      A. C. M. TRANSACTIONS ON MATHEMATICAL SOFTWARE V5 132-138.  
C  
      INTEGER A,P,IX,B15,B16,XHI,XALO,LEFTLO,FHI,K  
      DATA A/16807/,B15/32768/,B16/65536/,P/2147483647/  
      XHI=IX/B16  
      XALO=(IX-XHI*B16)*A  
      LEFTLO=XALO/B16  
      FHI=XHI*A+LEFTLO  
      K=FHI/B15  
      IX=(((XALO-LEFTLO*B16)-P)+(FHI-K*B15)*B16)+K  
      IF (IX.LT.0) IX=IX+P  
      RANDU=FLOAT(IX)*4.656612875E-10  
      RETURN  
      END  
C

      SUBROUTINE EQMAT(AMAT,BMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: EQUALIZATION OF MATRICES 
C     INPUT: AMAT, MATRIX DIMENSION L1 L2
C     OUPUT: BMAT, MATRIX EQUAL AMAT DIMENSION L1 L2
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*)
C
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    BMAT(I,J)=AMAT(I,J)
   99    CONTINUE
   98 CONTINUE
      RETURN
      END

      SUBROUTINE PRELIM(B0,D0,C4,ILX,B1,IMX,IP1,IP2,IS)
C*****************************************************************
C       PORPOSE: PROVIDE LNRC PRELIMINAR ESTIMATES
C     INPUT: B1, VECTOR OF ESTIMATES
C     OUTPUT: D0, VECTOR OF ESTIMATES NONMULTIPLICATIVE FORM
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B0(*),D0(*),C4(*)
      DIMENSION B1(*)
      EXTERNAL ZEROVEC,EQVEC
      I5=IP1+IP2*(IP1+1)+1
      CALL ZEROVEC(B0,LMAX,IP1+IP2)
      DO 117 I=1,IP1+1
 117     B0(I)=B1(I)
      DO 118 I=1,IP2
 118     B0(IP1+I+1)=B1((I-1)*(IP1+1)+IP1+2)
      CALL ZEROVEC(D0,LMAX,I5)
      IF(IP1.GT.0) THEN
	 DO 13 I=1,IP1+1
 13         D0(I)=B0(I)
      END IF
      IF (IP2.GT.0) THEN
	   CALL ZEROVEC(C4,LMAX,IP1+IP2+1)
	   C4(1)=1.0
	   DO 14 I=1,IP1+IP2
 14         C4(I+1)=B0(I+1)
	   DO 15 I1=1,IP2
	      DO 15 I2=1,IP1+1
 15         D0(IP1+(IP1+1)*(I1-1)+I2+1)=C4(I2)*C4(IP1+1+I1)
      END IF
      CALL EQVEC(D0,B0,IMX,I5)
      CALL ZEROVEC(D0,IMX,IP1+IS*IP2+1)
      IF (IP1.GT.0) THEN
	 DO 30 I=1,IP1+1
 30         D0(I)=B0(I)
      END IF
      IF (IP2.GT.0) THEN
	 DO 31 I1=1,IP2
	    DO 31 I2=1,IP1+1
 31            D0(IS*I1+I2)=B0(IP1+I2+(I1-1)*(IP1+1)+1)
      END IF
      RETURN
	END

      SUBROUTINE MULTEST(A2,D0,C2,A0,A,Q,B,B2,ELP,B9,B11,IMX,
     1                   IP1,IP2,IS,i9)

C*****************************************************************
C     PORPOSE: CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS 
C              PARTIAL DERIVATIVES MULTIPLICATIVE TESTING
C     INPUT: A2, VECTOR OF SCORES
C            D0, NON MULTIPLICATIVE VECTOR OF ESTIMATES
C            A, INFORMATION MATRIX
C     OUTPUT: A0, VECTOR OF ADJUSTING FACTORS
C             B, MATRIX OF RESTRICTED ESTIMATION
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A2(*),C2(*),D0(*),A0(*)
      DIMENSION Q(IMX,*),B(IMX,*),A(IMX,*),B2(IMX,*),ELP(IMX,*)
      DIMENSION B9(IMX,*),B11(IMX,*)
      EXTERNAL ZEROVEC,EQVEC,ZEROMAT,MULTMAT,MATINV,EQMAT
C
C       CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS PARTIAL
C       DERIVATIVES
C
      ik=1.0
      if (i9.eq.0) ik=-1.0
      I5=IP1+IP2*(IP1+1)+2
      CALL ZEROVEC(C2,IMX,I5+IP1*IP2)
	DO 50 I=1,I5
 50      C2(I)=A2(I)
      IF (IP1.GT.0.AND.IP2.GT.0) THEN
	   DO 52 I1=1,IP2
	      DO 52 I2=1,IP1
 52        C2(I5+IP1*(I1-1)+I2)=D0(IS*I1+I2+1)+D0(IS*I1+1)*D0(I2+1)*ik
c      write(3,*) (c2(iq),iq=1,i5+1)
      END IF
	CALL ZEROMAT(Q,IMX,I5,IP1*IP2)
      IF (IP1.GT.0.AND.IP2.GT.0) THEN
	DO 55 I1=1,IP1*IP2
	 Q(I5,I1)=0.0
 55      Q(1,I1)=0.0 
	   DO 56 I1=1,IP2
	      DO 56 I2=1,IP1
		 Q(I2+1,IP1*(I1-1)+I2)=D0(IS*I1+1)
		 Q(1+(IP1+1)*I1,IP1*(I1-1)+I2)=D0(I2+1)
 56        Q(1+(IP1+1)*I1+I2,IP1*(I1-1)+I2)=-1.0
      END IF
      CALL ZEROMAT(B,IMX,I5+IP1*IP2,I5+IP1*IP2)
      CALL ZEROMAT(B2,IMX,I5+IP1*IP2,I5+IP1*IP2)
	DO 60 I1=1,I5
	   DO 60 I2=I1,I5
	      B(I1,I2)=A(I1,I2)
 60         B(I2,I1)=A(I1,I2)
      DO 64 I1=1,I5
	   DO 64 I2=1,IP1*IP2
		B(I1,I5+I2)=Q(I1,I2)
 64         B(I2+I5,I1)=B(I1,I5+I2)
      CALL ZEROMAT(ELP,IMX,I5+IP1*IP2,I5+IP1*IP2)
      CALL MATINV(B,B2,ELP,IMX,I5+IP1*IP2,IFAIL)
      CALL EQMAT(B2,B,IMX,I5+IP1*IP2,I5+IP1*IP2)     
      CALL ZEROMAT(B9,IMX,IMX,1)
      CALL ZEROMAT(B11,IMX,IMX,1)
      DO 70 I=1,I5+IP1*IP2
 70      B9(I,1)=C2(I)
      CALL MULTMAT(B,B9,B11,IMX,IMX,I5+IP1*IP2,I5+IP1*IP2,1)
	CALL ZEROVEC(A0,IMX,I5+IP1*IP2)
	DO 72 I=1,I5+IP1*IP2
 72      A0(I)=B11(I,1)
      RETURN
      END

      SUBROUTINE MRQCOF(X,LX,SIG,NDATA,A,MA,ALPHA,BETA,
     1          NALP,d2yda,DYDA,XI,im)
C**************************************************************
C     THIS SUBROUTINE PROVIDES THE SCORES AND THE INFORMATION
C     MATRIX OF EACH REGIME.
C     INPUT: X IS THE MATRIX OF DATA EQUAL TO THE SAX MATRIX (OUTPUT OF
C            THE 'SORT' SUBROUTINE.
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS    
C            SIG IS THE PRELIMINAR ESTIMATOR OF SIGMA SQUARE
C            MA # OF PARAMETERS ( CONSTANT + AR ORDER ) ( WE USE MA+1 TO
C            INCLUDE SIGMA-SQUARE DERIVATIVES )
C     OUTPUT: ALPHA INFORMATION MATRIX
C             BETA VECTOR OF SCORES
C     EXTERNAL FUNCTIONS:
C     FUNCS EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES @ EACH
C             POINT
C**************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(LX,*),ALPHA(NALP,*),XI(*),D2YDA(nalp,1)
     1         ,A(*),BETA(*),DYDA(*)
      EXTERNAL ZEROVEC,ZEROMAT
      CALL ZEROVEC(BETA,NALP,nalp)
      CALL ZEROMAT(ALPHA,NALP,nalp,nalp)
      CALL ZEROMAT(D2YDA,NALP,nalp,nalp)    
      PF4=0.0
      if (im.eq.500) write(6,*) (a(ik),ik=1,ma)
      DO 15 I=1,NDATA
         CALL ZEROVEC(XI,NALP,nalp)
	 XI(MA)=X(I,MA+1)         
         DO 20 IQ=2,MA
 20         XI(MA+1-IQ)=X(I,IQ)
         Y=0.0
         Q=0.0
         R=0.0
         PA=0.0
         PB=0.0
         PC=0.0
         PD=0.0
         PE=0.0
         PF=0.0
         PF1=0.0
         PF2=0.0
         PF3=0.0
         CALL ZEROVEC(DYDA,NALP,nalp)
         CALL ZEROMAT(D2YDA,NALP,nalp,nalp)
         DO 1 J=2,MA
            Q=Q+(xi(ma-J+1)*A(J))
 1          R=R+((xi(ma-J+1)**2.0)*(A(J)**2.0))
         PF=xi(ma)-(A(1)+Q)
         pf1=xi(ma)-q
         pf2=pf1-2.0*a(1)
         pf3=pf1*pf2
C     *** derivative w.r.t constant ***
         DYDA(1)=PF/sig
C     *** derivatives w.r.t. AR parameters ***

         DO 2 K=2,ma
 2          DYDA(K)=(PF*xi(ma-K+1))/sig

C     *** second derivative w.r.t constant ***

         D2YDA(1,1)=-1.0/sig

C     *** second derivatives w.r.t constant and parameters ***

         DO 3 L=2,ma
 3          D2YDA(L,1)=-xi(ma-L+1)/sig

C     *** second derivative w.r.t constant and sigma square ***
 
         D2YDA(ma+1,1)=-PF/(sig**2.0)

C     *** second derivatives of parameters ***

         DO 4 L=2,ma
            DO 4 K=2,L
 4             D2YDA(L,K)=-(xi(ma-L+1)*xi(ma-K+1))/sig

C     *** second derivative of sigma-square ***

c      D2YDA(ma+1,ma+1)=(-sig+4.0*PE)/(-2.0*(sig**3.0))

C     *** second derivative of sigma-square and parameters ***

         DO 6 L=2,ma
 6          D2YDA(ma+1,L)=-(PF*xi(ma-L+1))/(sig**2.0)

         DO 14 J=1,MA+1
            DO 13 K=1,J
 13            ALPHA(J,K)=ALPHA(J,K)+D2YDA(J,K)
            BETA(J)=BETA(J)+DYDA(J)
 14      CONTINUE
         PF4=PF4+PF3
 15   CONTINUE 
      BETA(MA+1)=(-0.5*(REAL(NDATA)*(SIG-(A(1)**2.0))-PF4))/(SIG**2.0)
      DO 19 L=1,MA+1
 19       BETA(L)=BETA(L)/REAL(NDATA)
      DO 16 J=2,MA+1
	   DO 16 K=1,J-1
 16         ALPHA(K,J)=ALPHA(J,K)
      ALPHA(MA+1,MA+1)=(0.5*(REAL(NDATA)*(SIG-(2.0*(A(1)**2.0))-
     1        PF4)))/(SIG**3.0)
      DO 18 J=1,MA+1
	 DO 18 K=1,MA+1
 18         ALPHA(J,K)=-ALPHA(J,K)/REAL(NDATA)
      if (im.eq.500) write(6,*) ' ',NDATA
      if (im.eq.500) write(6,*) (beta(ik),ik=1,ma+1)
      RETURN
      END

      SUBROUTINE MATINV(AMAT,BMAT,ELP,ID1MAX,L1,IFAIL)
C************************************************************************
C     PORPOSE: MATRIX INVERSION
C     INPUT: AMAT, MATRIX DIMENSION L1,L1
C     OUTPUT: BMAT, MATRIX DIMENSION L1,L1
C             IFAIL, MONITORING VALUE
C************************************************************************

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*),ELP(ID1MAX,*)
      EXTERNAL EQMAT,ZEROMAT
C     
      CALL ZEROMAT(ELP,ID1MAX,L1,L1)
      CALL EQMAT(AMAT,ELP,ID1MAX,L1,L1)
      DO 115 I=1,L1
	 BMAT(I,I)=1.0
  115 CONTINUE
      DO 99 J=1,L1
	 DO 98 I=J,L1
	    IF (ELP(I,J).NE.0.0) GOTO 11
   98    CONTINUE
	 IFAIL=1
	 RETURN
   11    CONTINUE
      DO 97 K=1,L1
C
	 PIVOT01= ELP(J,K)
	 ELP(J,K)= ELP(I,K)
	 ELP(I,K)= PIVOT01
C
	 PIV= BMAT(J,K)
	 BMAT(J,K)=BMAT(I,K)
	 BMAT(I,K)= PIV
   97 CONTINUE
C
      IF(ELP(J,J).NE.0.0) GOTO 22
C
      IFAIL=1
      RETURN
C
C
   22 CONTINUE
      PPP= 1.0/ELP(J,J)
      DO 96 K=1,L1
         ELP(J,K)= ELP(J,K)*PPP
	 BMAT(J,K)= BMAT(J,K)*PPP
   96 CONTINUE
C
      DO 95 K=1,L1
C
	 IF(J.EQ.K) THEN
C
   33       CONTINUE
C
	 ELSE
	    PPP= ELP(K,J)
	    DO 94 L=1,L1
C
	       ELP(K,L)=ELP(K,L)-ELP(J,L)*PPP
	       BMAT(K,L)=BMAT(K,L)-BMAT(J,L)*PPP
   94       CONTINUE
C
	 END IF
   95 CONTINUE
   99 CONTINUE
C
      RETURN
      END


      SUBROUTINE GAP(A2,C2,C5,A0,ILX,A,Q,B,B2,ELP,B9,B11,IMX,IP1,
     1               IP2,IS)
C*****************************************************************
C       PORPOSE: CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS 
C              PARTIAL DERIVATIVES GAPS TESTING
C     INPUT: A2, VECTOR OF SCORES
C            A, INFORMATION MATRIX
C            IFS, FIRST ORDER GAP TESTING
C            IR, NUMBER OF ORDERS TO BE TESTED (BEGINS @ IFS)
C     OUTPUT: A0, VECTOR OF ADJUSTING FACTORS
C             B, MATRIX OF RESTRICTED ESTIMATION
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A2(*),C2(*),C5(*),A0(*)
      DIMENSION Q(IMX,*),B(IMX,*),A(IMX,*),B2(IMX,*),ELP(IMX,*)
      DIMENSION B9(IMX,*),B11(IMX,*)
      EXTERNAL ZEROVEC,EQVEC,ZEROMAT,MULTMAT,MATINV,EQMAT
C     INTRODUCTION OF SCORES
      KRP=IP1+IP2*IS
      IR=IS-IP1-1
      IL=IP1+IP2*(IP1+1)
      CALL ZEROVEC(C2,IMX,IL+IR)
      CALL ZEROVEC(C5,IMX,KRP)
      DO 10 I=1,KRP+2
 10      C2(I)=A2(I)
      C5(KRP+2)=C2(KRP+2)
C     PERMUTATION OF SCORE VECTOR
      DO 15 I1=1,IP2+1
	 DO 15 I=1,IP1+1
 15         C5(I+(I1-1)*(IP1+1))=C2(I+(I1-1)*IS) 
      DO 20 I1=1,IP2
	 DO 20 I=1,IR
 20         C5(1+IL+I+(I1-1)*IR)=C2(1+IP1+I+(I1-1)*IS)
      CALL ZEROMAT(B,IMX,KRP+1,KRP+1)
      CALL ZEROMAT(B2,IMX,KRP+1,KRP+1)
C     PERMUTATION OF THE INFORMATION MATRIX
      DO 50 I1=1,IP2+1
	 DO 50 I2=1,IP1+1
	    DO 50 I3=1,IP2+1
	       DO 50 I4=1,IP1+1
		  B(I2+(I1-1)*(IP1+1),I4+(I3-1)*(IP1+1))=
     1            A(I2+(I1-1)*IS,I4+(I3-1)*IS)
 50   CONTINUE
      DO 60 I1=1,IP2
	 DO 60 I2=1,IR
	    DO 60 I3=1,IP2
	       DO 60 I4=1,IR
		  B(1+IL+I2+(I1-1)*IR,1+IL+I4+(I3-1)*IR)=
     1            A(1+IP1+I2+(I1-1)*IR,1+IP1+I4+(I3-1)*IR)
 60   CONTINUE      
      DO 170 I1=1,KRP+2
	 B(KRP+2,I1)=A(KRP+2,I1)
 170     B(I1,KRP+2)=B(KRP+2,I1)
      CALL ZEROMAT(ELP,IMX,KRP+2,KRP+2)
      IFAIL=0
      CALL MATINV(B,B2,ELP,IMX,KRP+2,IFAIL)
c      write(6,*) ifail
      CALL EQMAT(B2,B,IMX,KRP+2,KRP+2)
c      do 172 il=1,krp+2
c 172     write(6,*) (b(il,ik),ik=1,krp+2)
      CALL ZEROMAT(B9,IMX,KRP+2,1)
      CALL ZEROMAT(B11,IMX,KRP+2,1)
      DO 190 I=1,KRP+2
 190     B9(I,1)=C5(I)
      CALL MULTMAT(B,B9,B11,IMX,IMX,KRP+2,KRP+2,1)
      CALL ZEROVEC(A0,IMX,KRP+2)
	DO 200 I=1,KRP+2
 200     A0(I)=B11(I,1)
      RETURN
      END

      SUBROUTINE ORDER(B,C,ID1MAX,IP1,IP2,IS,IA)
C************************************************************
C     PORPOSE: PERMUTATE A VECTOR   
C     INPUT: B, VECTOR TO BE PERMUTATED
C            IO, LENGTH OF THE VECTOR
C            IIFS, FIRST POSITION OF PERMUTATION
C            IIR, ORDERS TO BE PERMUTATED
C     OUTPUT: C, VECTOR PERMUTATED
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B(*),C(*)
C     PERMUTATION IN THE VECTOR
      IR=IS-IP1-1
      IL=IP1+IP2*(IP1+1)
      IF (IA.EQ.0) THEN
	 DO 10 I1=1,IP2+1
	    DO 10 I=1,IP1+1
 10            C(I+(I1-1)*(IP1+1))=B(I+(I1-1)*IS)
	 DO 20 I1=1,IP2
	    DO 20 I=1,IR
 20            C(1+IL+I+(I1-1)*IR)=B(1+IP1+I+(I1-1)*IS)
       ELSE IF(IA.EQ.1) THEN
	 DO 30 I1=1,IP2+1
	    DO 30 I=1,IP1+1
 30          C(I+(I1-1)*IS)=B(I+(I1-1)*(IP1+1))
	 DO 40 I1=1,IP2
	    DO 40 I=1,IR
 40          C(1+IP1+I+(I1-1)*IS)=B(1+IL+I+(I1-1)*IR)
      END IF
      RETURN
      END


