C*************************************************************************
C       FORTRAN77 code
C       Coded by: Antoni Vidiella-i-Anguera
C       File: Seasest.for
C
C       ESTIMATION PROGRAM FOR SEASONAL SETAR MODELS 
C       CORRECTED SELECTION CRITERIA
C       N    = SAMPLE SIZE OF THE SERIES
C       ID1  = DEGREE OF DIFFERENCING ( APPLIED OVER TRANSFORMED DATA!)
C       ID2  = DEGREE OF SEASONAL DIFFERENCING
C       IP1M = ORDER OF NON.SEASONAL MAX AR OPERATOR, IF IP1=0 IP1=IS-1
C       IP2  = ORDER OF SEASONAL AR OPERATOR
C       B0   = INITIAL ESTIMATE VECTOR
C       B1   = FINAL ESTIMATE VECTOR
C       IS   = PERIOD OF SEASONALITY
C       S2   = ESTIMATE OF RESIDUAL VARIANCE
C       I0   = MAXIMUM NUMBER OF ITERATIONS
C       ISEL = SELECTION CRITERIA (0 AICC, 1 AICU)
C       ITR  =  0, USE RAW DATA
C           1, USE LOGARITHM OF DATA
C           2, USE LOG 1ST DIFFERENCES OF DATA
C           3, USE RAW (NO LOG) 1ST DIFFERENCES OF DATA
C           4, USE LOG-LINEAR DETRENDED DATA
C           5, USE RAW LINEAR DETRENDED DATA
C
C     L1 = NUMBER OF REGIONS
C     LINEARITY TESTING
C     ESTIMATION OF THE GAPS MODEL OR THE SEASONAL MODEL (LM TEST) 
C     METHOD OF ESTIMATION: SCORING & LEAST SQUARES
C************************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER THMAX
C
C     *** CHARACTER VARIABLES ***
C
      CHARACTER*12 FILEI,FILEO
      CHARACTER*50 DESC
C
C     *** PARAMETER VARIABLES ***
C
      PARAMETER (LMAX=40,NMAX=500,THMAX=1)
C
C     *** REAL VECTORS AND MATRICES ***
C      
      DIMENSION B0(LMAX),B1(LMAX),A1(LMAX),Y(NMAX),B11(LMAX,1)
      DIMENSION C(LMAX),D0(LMAX),ELP(LMAX,LMAX),AC2(LMAX),SDM2(LMAX)
      DIMENSION A2(LMAX),A(LMAX,LMAX),B(LMAX,LMAX),Q(LMAX,LMAX)
      DIMENSION C2(LMAX),A0(LMAX),B9(LMAX,1),AC1(LMAX),DENORM(NMAX)
      DIMENSION Q1(LMAX,LMAX),B2(LMAX,LMAX),SDM(LMAX),ZLN(2,LMAX)
      DIMENSION AX(NMAX,LMAX,THMAX+1),THD(THMAX),KKARAY(THMAX+1)
      DIMENSION NOBS(THMAX+1),TIME(NMAX),C5(LMAX),SAX(NMAX,LMAX)
      DIMENSION AB0(LMAX),AB1(LMAX),IPAZ(LMAX),H8(LMAX)
      DIMENSION SAXI(NMAX,LMAX),SAG(NMAX,LMAX),Y1(NMAX),CMATX2(2,LMAX)
      DIMENSION I2PAZ(LMAX),CMATX(2,LMAX),SIMV(NMAX),ANORM(2)
      DIMENSION IXT1(2),IXT2(2)
C
C     *** EXTERNAL SUBROUTINES ***
C
      EXTERNAL ZEROMAT,ZEROVEC,EQMAT,EQVEC,MATINV
      EXTERNAL SORT,COVAR,IZEROVEC,ZEROMAT3,DETREND
      EXTERNAL GAP,PRELIM,MULTEST,ORDER,HUSHLD,NFR
      EXTERNAL ARMFIT,VARCOV,MRQCOF,FUNCS,LINMUD
C
C     READ INPUT FILE
C
      OPEN (7,FILE='infest.dat',STATUS='UNKNOWN')
      READ (7,9020) DESC
      READ (7,*) N
      READ (7,*) ID1
      READ (7,*) ID2
      READ (7,*) IIP1,IIP2
      READ (7,*) IPAZ(1),I2PAZ(1),IPAZ(2),I2PAZ(2)
      READ (7,*) IS
      READ (7,*) I0
      READ (7,*) ITR
      READ (7,*) THDX
      READ (7,*) ID
      READ (7,*) ISEL
      READ (7,9021) FILEO
      READ (7,9021) FILEI
      IF (IIP1.LT.0) THEN
         WRITE (6,*) 'Error: P1 of AR(p) is negative!!'
         STOP
      END IF
      IF (IIP2.LT.0) THEN
         WRITE(6,*) 'Error: P2 of SAR(p) is negative!!'
        STOP
      END IF
C
C     READ IN DATA FROM FILE "FILEI"
C
      OPEN(9,FILE=FILEI,STATUS='UNKNOWN')
      DO 12,IT=1,N
           READ(9,*) Y(IT)
 12    CONTINUE 
C
C     *** OPEN OUTPUT FILE ***
C
        OPEN (8,FILE=FILEO,STATUS='UNKNOWN')
C
C     *** LIMITING NUMBER OF ITERATIONS
C
        IF (I0.GT.70) I0=70
C
C     *** DATA TRANSFORMATIONS ***
C
      IF ((ITR.EQ.1).OR.(ITR.EQ.2).OR.(ITR.EQ.4)) THEN
        DO 11 M=1,N
 11        Y(M)=LOG(Y(M))
      END IF
      IF ((ITR.EQ.2).OR.(ITR.EQ.3)) THEN
        N = N-1
        DO 6 M=1,N
 6          Y(M)=Y(M+1)-Y(M)
      END IF
      IF ((ITR.EQ.4).OR.(ITR.EQ.5)) THEN
        DO 7 M=1,N
 7         TIME(M)=REAL(M)
        CALL DETREND(Y,N,TIME)
      END IF
C
C     *** DIFFERENCING & SEASONAL DIFFERENCING ***
C
      IF (ID1.NE.0) THEN
         DO 1 I3=1, ID1
              N = N-1
            DO 1 I4=1, N
 1             Y(I4) = Y(I4+1) - Y(I4)
      END IF
      IF (ID2.NE.0) THEN
           DO 3 I3=1, ID2
              N = N - IS
              DO 4 I4=1, N
                   Y(I4) = Y(I4+IS) - Y(I4)
  4         CONTINUE
  3      CONTINUE
      END IF
      CALL EQVEC(Y,Y1,NMAX,N)
      N1=N
      IS1=IS
      IR1=IR
      IK1=IPAZ(1)+I2PAZ(1)*IS
      IK2=IPAZ(2)+I2PAZ(2)*IS
      IK3=IIP1+IIP2*IS
      IP1T=IPAZ(1)
      IP2T=I2PAZ(1)
      IF(IK2.GT.IK1) THEN
         IP1T=IPAZ(2)
         IP2T=I2PAZ(2)
      END IF
      IF (IK3.GT.IK2.and.IK3.GT.IK1) THEN
         IP1T=IIP1
         IP2T=IIP2
      END IF


C**********************************************************************
C     ESTIMATION OF THE FIRST MODEL: AR - TEST OF GAPS
C     
C**********************************************************************
C *** INICIALIZATION
      S0=0.0
      IER=0
      IES=0
      IND=1
      THD(1)=0.0
      KKARAY(1)=IP1T+IP2T*IS
      ITF=KKARAY(1)
      ICP=1
      SIGT=0.0
      IP1=IIP1
      IP2=IIP2
      I5=1+IP1+IP2*IS
      I9=0
      KRL=IP1+1
      IL=IP1+IP2*KRL
      KRP=I5-1
C
C     COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE GAPS MODEL
C


      CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
      CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,0,ID,NOBS,IER,IES,is)
      CALL COVAR(AX,NMAX,LMAX,N,ITF,KKARAY(1),IS,ICP,S0,C,lmax)
        N=NOBS(1)
      N2=N


 1900  IF (I9.GT.0) GOTO 2300
      K1=I5+1 
      DO 23 I1=1,N
           DO 26 I2=1,I5
 26         SAX(I1,I2)=AX(I1,I2,1)
 23       SAX(I1,K1)=AX(I1,ITF+2,1)
      CALL EQMAT(SAX,SAXI,NMAX,N,K1)
      CALL HUSHLD(SAX,N,K1,NMAX)
      CALL ARMFIT(SAX,NMAX,AC1,I5,N,VA,IMIN,AICC,AICU)
      CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
      IF (IND.GT.0) THEN
         DO 31 I1=1,N
            DO 41 I3=1,IP2+1
                DO 41 I2=1,KRL
 41               SAG(I1,(I3-1)*KRL+I2)=AX(I1,IS*(I3-1)+I2,ICP)
 31         SAG(I1,(I3-1)*KRL+1)=AX(I1,ITF+2,ICP)
            CALL HUSHLD(SAG,N,IL+2,NMAX)
            CALL ARMFIT(SAG,NMAX,AC2,IL+1,N,VA2,IMIN,AICC2,AICU2)          
            CALL VARCOV(SAG,NMAX,IMIN,VA2,SDM2)
      END IF
      DO 27 I1=1,I5
 27      B1(I1)=AC1(I1)
      S2=VA
      IF (IND.EQ.0) THEN
           CALL ZEROVEC(B0,LMAX,I5)
           DO 29 I1=1,I5
 29           A1(I1)=SDM(I1)
           GOTO 2200
      END IF
 2300 CONTINUE
      CALL EQVEC(B1,B0,LMAX,I5)
      CALL MRQCOF(SAXI,NMAX,S2,N,B1,I5,A,A2,LMAX,FUNCS)
      CALL GAP(A2,C2,C5,A0,LMAX,A,Q,B,B2,
     1            ELP,B9,B11,LMAX,IP1,IP2,IS)
      D9=ABS(A0(1))
      DO 74 I=1,K1
 74      IF(D9.LE.ABS(A0(I))) D9=ABS(A0(I))
        CALL ZEROVEC(B1,LMAX,K1)
      CALL ZEROVEC(AB0,LMAX,K1)
      CALL ORDER(B0,AB0,LMAX,IP1,IP2,IS,0)
      DO 76 I=1,I5
 76      B1(I)=AB0(I)+A0(I)
      S2=S2+A0(K1)
      IF(D9.LE.(1.D-3).OR.I9.EQ.I0) THEN
           CALL EQVEC(AB0,B0,LMAX,I5)
         DO 75 I=1,I5
 75         CMATX(1,I)=B1(I)
           GOTO 2800
      END IF
      CALL ZEROVEC(AB1,LMAX,I5)
      CALL ORDER(B1,AB1,LMAX,IP1,IP2,IS,1)
      CALL EQVEC(AB1,B1,LMAX,I5)
      I9=I9+1
      GOTO 1900
 2800 CALL ZEROVEC(A1,LMAX,I5)
      DO 84 I=1,IL+1
 84      A1(I)=SQRT(B(I,I)/REAL(N))
      C3=0.0
      DO 90 I1=IL+1,I5
         DO 90 I2=IL+1,I5
 90         C3=C3+B1(I1)*B1(I2)*B(I1,I2)
      C3=N*C3
      AICC=N*LOG(S2)+((2*N*(I5+2))/(N-I5-3))
      AICU=AICC+N*LOG(N/REAL(N-I5-2))
 2200 CONTINUE
C
C      WRITING INITIAL INFO
C
      WRITE(8,9000) DESC
      WRITE(8,9001) ID1
      WRITE(8,9002) ID2
      WRITE(8,9003) N1
      WRITE(8,9004) IS
      WRITE(8,9006)
      IF (ISEL.EQ.0) WRITE(8,9030)
      IF (ISEL.EQ.1) WRITE(8,9031)
      WRITE(8,9007) S0,C(1)
      WRITE(8,9008) ID
      WRITE(8,*) ' '
      WRITE(8,9032)
C     *** WRITING RESULTS ***
      PAICC=0.0
      PAICU=0.0
      ixt1(1)=ip1t
      ixt2(1)=ip2t
      CALL WRITEGAP(B1,A1,B0,I9,IP1,IP2,IS,S2,AICC,AICU,
     1            N,1,PAICC,PAICU,C3,IND)
      IF (IND.GT.0) CALL WRGP(ac2,sdm2,va2,AICC2,
     1        AICU2,IP1,IP2,IS,0,pAICC,pAICU) 
      CALL NFR(Y1,N1,IES,THD(1),0,ID,CMATX,NOBS,
     1    IXT1,ixt2,IS,SIMV,DENORM,ANORM,NMAX)
      CALL LINMUD(SIMV,N1-IES)
      WRITE(8,9039)
      CALL ZEROVEC(SIMV,NMAX,N1-IES)
C**********************************************************************
C     ESTIMATION OF THE SECOND MODEL: TAR - FINDING THE BEST THRESHOLD
C                                           TEST GAPS IN EACH PIECE
C**********************************************************************
C *** INICIALIZATION
      NC=1
      PAICC=0.0
      PAICU=0.0
      PAICC2=0.0
      PAICU2=0.0
      IC=0
      N=N1
      IND=1
      IER=0
      IES=0
      ikp=0
      SIGTNL=0.0
      THD(1)=THDX
      WRITE(8,9015) THD(1)
      DO 186 I=1,2
 186     KKARAY(I)=IP1T+IS*IP2T
      ITF=KKARAY(1)
      CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
      CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,1,ID,NOBS,IER,IES,is)
      DO 195 ICP=1,2
         if(nobs(icp).lt.20) then
            ikp=1
            goto 195
         end if
         IP2=I2PAZ(ICP)
         IP1=IPAZ(ICP)
         KR=IP1+IS*IP2
         KRL=1+IP1
         N=NOBS(ICP)
         K1=KR+2
         K2=KR+1
         S2=0.0
         I5=K2
         I9=0
         IL=IP1+IP2*KRL
 3000    IF (I9.GT.0) GOTO 3300
C
C       COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE GAPS MODEL
C
         CALL ZEROMAT(SAX,NMAX,N,K1)
         CALL ZEROMAT(SAG,NMAX,N,K1)
         CALL ZEROMAT(SAXI,NMAX,N,K1)
         CALL ZEROVEC(SDM,LMAX,K2)
         DO 101 I1=1,N
            DO 100 I2=1,K2
 100           SAX(I1,I2)=AX(I1,I2,ICP)
 101        SAX(I1,K1)=AX(I1,ITF+2,ICP)   
         CALL EQMAT(SAX,SAXI,NMAX,N,K1)
         CALL HUSHLD(SAX,N,K1,NMAX)
         CALL ARMFIT(SAX,NMAX,AC1,K2,N,VA,IMIN,AICC,AICU)
         CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
         IF (IND.GT.0) THEN
            DO 139 I1=1,N
               DO 141 I3=1,IP2+1
                  DO 141 I2=1,KRL
 141     SAG(I1,(I3-1)*KRL+I2)=AX(I1,IS*(I3-1)+I2,ICP)
 139     SAG(I1,(I3-1)*KRL+1)=AX(I1,ITF+2,ICP)
         CALL HUSHLD(SAG,N,IL+2,NMAX)
         CALL ARMFIT(SAG,NMAX,AC2,IL+1,N,VA2,IMIN,AICC2,AICU2)
         CALL VARCOV(SAG,NMAX,IMIN,VA2,SDM2)
         DO 142 I=1,IL+1
 142        CMATX2(ICP,I)=AC2(I)
         END IF
         DO 110 I1=1,I5
 110        B1(I1)=AC1(I1)
         S2=VA
         IF (IND.EQ.0) THEN
            CALL ZEROVEC(B0,LMAX,I5)
            DO 111 I1=1,I5
 111           A1(I1)=SDM(I1)
               GOTO 3200
            END IF
 3300    CONTINUE
         CALL EQVEC(B1,B0,LMAX,I5)
         CALL MRQCOF(SAXI,NMAX,S2,N,B1,K2,A,A2,LMAX,FUNCS)
         CALL GAP(A2,C2,C5,A0,LMAX,A,Q,B,B2,
     1      ELP,B9,B11,LMAX,IP1,IP2,IS)
         D9=ABS(A0(1))
         DO 170 I=1,I5+1
 170     IF(D9.LE.ABS(A0(I))) D9=ABS(A0(I))
         CALL ZEROVEC(B1,LMAX,I5+1)
         CALL ZEROVEC(AB0,LMAX,I5+1)
         CALL ORDER(B0,AB0,LMAX,IP1,IP2,IS,0)
         DO 180 I=1,I5
 180        B1(I)=AB0(I)+A0(I)
            S2=S2+A0(I5+1)
            IF(D9.LE.(1.D-3).OR.I9.EQ.I0) THEN
               CALL EQVEC(AB0,B0,LMAX,I5)
               GOTO 3800
            END IF
            CALL ZEROVEC(AB1,LMAX,I5)
            CALL ORDER(B1,AB1,LMAX,IP1,IP2,IS,1)
            CALL EQVEC(AB1,B1,LMAX,I5)
            I9=I9+1
            GOTO 3000
 3800    CONTINUE
         DO 176 I2=1,I5
 176        CMATX(ICP,I2)=B1(I2)  
         CALL ZEROVEC(A1,LMAX,IL+1)
         DO 210 I=1,IL+1
 210        A1(I)=SQRT(B(I,I)/REAL(N))
         C3=0.0
         DO 230 I1=IL+1,K2
            DO 230 I2=IL+1,K2
 230           C3=C3+B1(I1)*B1(I2)*B(I1,I2)
            C3=N*C3
         AICC=N*LOG(S2)+((2*N*(I5+2))/(N-I5-3))
         AICU=AICC+N*LOG(DBLE(N/REAL(N-I5-2)))
 3200    CONTINUE
         PAICC=PAICC+AICC
         PAICC2=PAICC2+AICC2
         PAICU=PAICU+AICU
         PAICU2=PAICU2+AICU2
C     *** WRITING RESULTS ***
         WRITE(8,*) ' '
         WRITE(8,9033)
         CALL WRITEGAP(B1,A1,B0,I9,IP1,IP2,IS,S2,AICC,AICU
     1            ,NOBS(ICP),NC,PAICC,PAICU,C3,IND)
         IF (IND.GT.0) CALL WRGP(AC2,SDM2,VA2,AICC2,
     1        AICU2,IP1,IP2,IS,NC,PAICC2,PAICU2)
         NC=NC+1
         AICC=0.0
         AICU=0.0
 195  CONTINUE
      if (ikp.eq.1) goto 196
      CALL NFR(Y1,N1,IES,THD(1),1,ID,CMATX2,NOBS,
     1    IPAZ,I2PAZ,IS,SIMV,DENORM,ANORM,NMAX)
      CALL LINMUD(SIMV,N1-IES)
      CALL ZEROVEC(SIMV,NMAX,N1-IES)
      CALL NFR(Y1,N1,IES,THD(1),1,ID,CMATX,NOBS,
     1    IPAZ,I2PAZ,IS,SIMV,DENORM,ANORM,NMAX)
      CALL LINMUD(SIMV,N1-IES)
      CALL ZEROVEC(SIMV,NMAX,N1-IES)
      WRITE(8,9039)
 196  continue
C*****************************************************************
C      ESTIMATION OF THE THIRD MODEL: TAR-SAR IN EACH PIECE
C      TESTING: LM  FOR SEASONALITY
C*****************************************************************       
      N=N1
      NC=1
      PAICC=0.0
      PAICU=0.0
      IER=0
      IES=0
      ikp=0
      IS=IS1
      CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
      CALL IZEROVEC(KKARAY,THMAX+1,2)
      DO 302 I=1,2
 302     KKARAY(I)=IPAZ(I)+IS*I2PAZ(I)
      CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,1,ID,NOBS,IER,IES,is)
C      *** BEGIN LOOP PIECES TO BE ESTIMATED ***
      DO 310 ICP=1,2
C      *** ASIGNATION OF VALUES ***
         if(nobs(icp).lt.20) then
            ikp=1
            goto 310
         end if
         IR=1
         N=NOBS(ICP)
         KARAY=KKARAY(ICP)
         S0=0.0
         S2=0.0
         IP2=I2PAZ(ICP)
         IP1=IPAZ(ICP)
         I5=IP1+IP2*(IP1+1)
         I9=0
         IF(IP1*IP2.EQ.0) IR=0
 4000    IF (I9.GT.0) GOTO 4300
C
C       COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE SEASONAL MODEL
C
         K1=IP1+IP2*(IP1+1)+2
         K2=IP1+IP2*(IP1+1)+1
         CALL ZEROVEC(B1,LMAX,K2)          
         CALL ZEROMAT(SAX,NMAX,N,K1)
         CALL ZEROMAT(SAXI,NMAX,N,K1)
         CALL ZEROVEC(SDM,LMAX,K2)
         DO 320 I1=1,N
            DO 321 I2=1,IP1+1
 321           SAX(I1,I2)=AX(I1,I2,ICP)
               DO 322 I3=1,IP2
                  DO 322 I4=1,IP1+1
 322              SAX(I1,1+IP1+I4+(I3-1)*(IP1+1))=AX(I1,I4+IS*I3,ICP)
 320     SAX(I1,IP1+IP2*(IP1+1)+2)=AX(I1,IP1+IP2*IS+2,ICP)
         CALL EQMAT(SAX,SAXI,NMAX,N,K1)
         CALL HUSHLD(SAX,N,K1,NMAX)
         CALL ARMFIT(SAX,NMAX,AC1,K2,N,VA,IMIN,AICC,AICU)
         CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
         DO 330 I1=1,K2
 330        B1(I1)=AC1(I1)
         S2=VA 
         CALL EQVEC(B1,B0,LMAX,K2)
         IF (IND.EQ.0) THEN
            DO 340 I1=1,K2
 340           A1(I1)=SDM(I1)
               CALL ZEROVEC(D0,LMAX,IP1+IS*IP2)
               IF (IP1.GT.0) THEN
                  DO 360 I=1,IP1+1
 360                 D0(I)=B1(I)
               END IF
               IF (IP2.GT.0) THEN
                  DO 370 I1=1,IP2
                     DO 370 I2=1,IP1+1
 370                    D0(IS*I1+I2)=B1(IP1+I2+(I1-1)*(IP1+1)+1)
               END IF
               GOTO 4200
            END IF
C     *** INTRODUCTION PRELIMINAR PARAMETERS TEST ****************
            CALL ZEROVEC(C5,LMAX,I5+2)
            CALL PRELIM(B0,D0,C5,LMAX,B1,LMAX,IP1,IP2,IS)
            CALL EQVEC(B0,B1,LMAX,I5+1)
 4300     CONTINUE
            CALL MRQCOF(SAXI,NMAX,S2,N,B1,K2,A,A2,LMAX,FUNCS)
C
C           MULTIPLICATIVE TEST LM 
C
          CALL MULTEST(A2,D0,C2,A0,LMAX,A,Q,B,B2,ELP,B9,B11,LMAX,
     1                   IP1,IP2,IS)
          D9=ABS(A0(1))
          DO 410 I=1,I5+2
 410       IF(D9.LE.ABS(A0(I))) D9=ABS(A0(I))
          CALL ZEROVEC(B1,LMAX,I5+1)
          DO 420 I=1,I5+1
 420          B1(I)=B0(I)+A0(I)
          S2=S2+A0(I5+2)
          IF(D9.LE.(1.D-3).OR.I9.EQ.I0) GOTO 4800
          CALL EQVEC(B1,B0,LMAX,I5+1)
          IF (IP1.GT.0) THEN
             DO 430 I=1,IP1+1
 430          D0(I)=B0(I)
          END IF
          IF (IP2.GT.0) THEN
             DO 440 I1=1,IP2
                DO 440 I2=1,IP1+1
 440             D0(IS*I1+I2)=B0(IP1+I2+(I1-1)*(IP1+1)+1)
          END IF
          I9=I9+1
          GOTO 4000
 4800   CONTINUE
          DO 476 I2=1,I5+1
 476          ZLN(ICP,I2)=B1(I2)  
          CALL ZEROVEC(A1,LMAX,I5+1)
          DO 450 I=1,I5+1
 450        A1(I)=SQRT(B(I,I)/N)
          CALL ZEROMAT(Q,LMAX,IP1*IP2,IP1*IP2)
          CALL ZEROMAT(Q1,LMAX,IP1*IP2,IP1*IP2)
            DO 460 I1=1,IP1*IP2
               DO 460 I2=I1,IP1*IP2
                    Q(I1,I2)=-B(I5+2+I1,I5+2+I2)
 460            Q(I2,I1)=Q(I1,I2)
          CALL ZEROMAT(ELP,LMAX,IP1*IP2,IP1*IP2)
          CALL MATINV(Q,Q1,ELP,LMAX,IP1*IP2,IFAIL)
          C3=0.0
            DO 470 I1=1,IP1*IP2
               DO 470 I2=1,IP1*IP2
 470            C3=C3+A0(I5+I1+2)*A0(I5+I2+2)*Q1(I1,I2)
         C3=N*C3
       AICC=N*LOG(S2)+((2*N*(I5+2))/(N-I5-3))
       AICU=AICC+N*LOG(DBLE(N/REAL(N-I5-2)))
 4200  CONTINUE
       PAICC=PAICC+AICC
       PAICU=PAICU+AICU
       WRITE(8,*) ' '
       WRITE(8,9034)
       CALL WRMULT(B1,A1,B0,D0,I9,IP1,IP2,IS,S2,
     1         AICC,AICU,IR,NOBS(ICP),NC,PAICC,PAICU,C3)
       NC=NC+1
       AICC=0.0
       AICU=0.0
 310   CONTINUE
c       if (ikp.eq.1) goto 777
       DO 327 I3=1,2
          DO 337 I=1,IPAZ(I3)+1
 337         ZLN(I3,I)=ZLN(I3,I)
          DO 328 I=1,I2PAZ(I3)
 328      ZLN(I3,IPAZ(I3)+I+1)=ZLN(I3,(I-1)*(IPAZ(I3)+1)+IPAZ(I3)+2)
          IF (IPAZ(I3).GT.0) THEN
             DO 333 I=1,IPAZ(I3)+1
 333            CMATX(I3,I)=ZLN(I3,I)
          END IF
          IF (I2PAZ(I3).GT.0) THEN
             CALL ZEROVEC(H8,LMAX,IPAZ(I3)+I2PAZ(I3)+1)
             H8(1)=1.0
             DO 334 I=1,IPAZ(I3)+I2PAZ(I3)
 334             H8(I+1)=ZLN(I3,I+1)
             DO 335 I1=1,I2PAZ(I3)
                DO 335 I2=1,IPAZ(I3)+1
 335      CMATX(I3,IPAZ(I3)+(IPAZ(I3)+1)*(I1-1)+I2+1)=
     1                  H8(I2)*H8(IPAZ(I3)+1+I1)
          END IF         
 327      CONTINUE
          CALL ZEROVEC(DENORM,NMAX,NMAX)
          CALL ZEROVEC(ANORM,2,2)          
          CALL NFR(Y1,N1,IES,THD(1),0,ID,CMATX,NOBS,
     1    IPAZ,I2PAZ,IS,SIMV,DENORM,ANORM,NMAX)
          CALL LINMUD(SIMV,N1-IES)            
       WRITE(8,9039)
 777   continue
C      We skip estimation of fourth and fifth models ( even no diff! )
       goto 1190
C*****************************************************************
C      ESTIMATION OF THE FOURTH MODEL: TAR-AR IN FIRST PIECE
C                                     TSAR IN SECOND PIECE
C      TESTING: LM GAPS TESTING IN FIRST PIECE
C               NO POSSIBLE TESTING IN SECOND PIECE
C*****************************************************************       
       N=N1
       NC=1
       PAICC=0.0
       PAICU=0.0
       PAICC2=0.0
       PAICU2=0.0
       CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
       CALL IZEROVEC(KKARAY,THMAX+1,2)
       KKARAY(1)=IPAZ(1)+I2PAZ(1)*IS
       KKARAY(2)=MAX0(I2PAZ(2),1)*IS
       CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,1,ID,NOBS,IER,IES,is)
C      *** BEGIN LOOP PIECES TO BE ESTIMATED ***
       DO 900 ICP=1,2
C      *** ASIGNATION OF VALUES ***
       N=NOBS(ICP)
       IP1=IPAZ(ICP)
       IP2=I2PAZ(ICP)
       IF (ICP.EQ.2.AND.IP2.EQ.0) IP2=1
       KR=KKARAY(ICP)
       KRL=IP1+1
       K1=KR+2
       K2=KR+1
       S2=0.0
       I9=0
       IL=IP1+IP2*KRL
C
C       COMPUTATION OF THE SCORE VECTOR AND INFORMATION MATRIX
C
 7000     IF (I9.GT.0) GOTO 7300
C
C       COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE MODEL
C
       CALL ZEROVEC(D0,LMAX,LMAX)
       CALL ZEROMAT(SAX,NMAX,NMAX,LMAX)
       CALL ZEROMAT(SAXI,NMAX,NMAX,LMAX)
       CALL ZEROMAT(A,LMAX,LMAX,LMAX)
       CALL ZEROVEC(SDM,LMAX,LMAX)
       CALL ZEROVEC(AC1,LMAX,LMAX)
       CALL ZEROVEC(A2,LMAX,LMAX)
       CALL ZEROVEC(C2,LMAX,LMAX)
       CALL ZEROVEC(B0,LMAX,LMAX)
       CALL ZEROVEC(B1,LMAX,LMAX)
       CALL ZEROMAT(B,LMAX,LMAX,LMAX)
       CALL ZEROMAT(B2,LMAX,LMAX,LMAX)
       CALL ZEROMAT(B9,LMAX,LMAX,1)
       CALL ZEROMAT(B11,LMAX,LMAX,1)
       CALL ZEROMAT(Q,LMAX,LMAX,1)
       CALL ZEROVEC(A0,LMAX,LMAX)
       IF (ICP.EQ.2) THEN
          DO 925 I1=1,N
             DO 935 I2=1,IP2+1
 935            SAX(I1,I2)=AX(I1,(I2-1)*IS+1,ICP)
 925         SAX(I1,IP2+2)=AX(I1,IS*IP2+2,ICP)
          K1=IP2+2
          K2=IP2+1
          IP1=0
          CALL EQMAT(SAX,SAXI,NMAX,N,K1)
          ELSE IF(ICP.EQ.1) THEN
             DO 905 I1=1,N
                DO 905 I2=1,K1
                   SAX(I1,I2)=AX(I1,I2,ICP)
 905               SAXI(I1,I2)=SAX(I1,I2)
          END IF
          CALL HUSHLD(SAX,N,K1,NMAX)
          CALL ARMFIT(SAX,NMAX,AC1,K2,N,VA,IMIN,AICC,AICU)
          CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
          IF (ICP.EQ.1.AND.IND.GT.0) THEN
             DO 931 I1=1,N
                DO 941 I3=1,IP2+1
                   DO 941 I2=1,KRL
 941               SAG(I1,(I3-1)*KRL+I2)=AX(I1,IS*(I3-1)+I2,ICP)
 931      SAG(I1,(I3-1)*KRL+1)=AX(I1,K1,ICP)
          call hushld(sag,n,il+2,nmax)
          call armfit(sag,nmax,ac2,il+1,n,va2,imin,AICC2,AICU2)
          call varcov(sag,nmax,imin,va2,sdm2)
       end if
       DO 910 I1=1,K2
 910   B1(I1)=AC1(I1)
       S2=VA
       CALL EQVEC(B1,B0,LMAX,K2)
       IF (IND.EQ.0.OR.ICP.EQ.2) THEN
          CALL ZEROVEC(B0,LMAX,K2)
          DO 915 I1=1,K2
 915         A1(I1)=SDM(I1)
             CALL EQVEC(B1,B0,LMAX,K2)
             CALL ZEROVEC(D0,LMAX,IP1+IS*IP2+1)
             IF (IP1.GT.0) THEN
                DO 920 I=1,IP1+1
 920               D0(I)=B0(I)
             END IF
          GOTO 7200
       END IF
 7300  CONTINUE
       CALL MRQCOF(SAXI,NMAX,S2,N,B1,K2,A,A2,LMAX,FUNCS)
          IF (ICP.EQ.1) CALL GAP(A2,C2,C5,A0,LMAX,A,Q,B,B2,ELP,B9,
     1        B11,LMAX,IP1,IP2,IS)
          D9=ABS(A0(1))
          DO 1000 I=1,K1
 1000      IF(D9.LE.ABS(A0(I))) D9=ABS(A0(I))
          CALL ZEROVEC(B1,LMAX,K2)
          CALL ZEROVEC(AB0,LMAX,K2)
          CALL ORDER(B0,AB0,LMAX,IP1,IP2,IS,0)
          DO 1005 I=1,K2
 1005      B1(I)=AB0(I)+A0(I)
          S2=S2+A0(K1)
          IF(D9.LE.(1.D-3).OR.I9.EQ.I0) THEN
             CALL EQVEC(AB0,B0,LMAX,K2)
             GOTO 7800
          END IF
          CALL ZEROVEC(AB1,LMAX,K2)
          CALL ORDER(B1,AB1,LMAX,IP1,IP2,IS,1)
          CALL EQVEC (AB1,B0,LMAX,K2)
          IF (IP1.GT.0) THEN
             DO 1020 I=1,IP1+1
 1020           D0(I)=B0(I)
          END IF
          I9=I9+1
          GOTO 7000
 7800   CALL ZEROVEC(A1,LMAX,K2)
          DO 1030 I=1,IL+1
 1030      A1(I)=SQRT(B(I,I)/N)
          C3=0.0
          DO 1050 I1=IL+1,K2
               DO 1050 I2=IL+1,K2
 1050           C3=C3+B1(I1)*B1(I2)*B(I1,I2)
          C3=N*C3
        AICC=N*LOG(S2)+((2*N*(K2+2))/(N-K2-3))
        AICU=AICC+N*LOG(DBLE(N/REAL(N-K2-2)))
 7200   CONTINUE
          PAICC=PAICC+AICC
          PAICU=PAICU+AICU
          if (icp.eq.2) then
             AICC2=AICC
             AICU2=AICU
          end if
          pAICC2=pAICC2+AICC2
          pAICU2=pAICU2+AICU2
          WRITE(8,*) ' '
          WRITE(8,9037)
          IF (ICP.EQ.2) THEN
             CALL WRMULT(B1,A1,B0,D0,I9,IP1,IP2,IS,S2,
     1               AICC,AICU,IND,NOBS(ICP),NC,PAICC,PAICU,C3)
          ELSE IF(ICP.EQ.1) THEN
            CALL WRITEGAP(B1,A1,B0,I9,IP1,IP2,IS,S2,AICC,AICU
     1           ,NOBS(ICP),NC,PAICC,PAICU,C3,IND)
        IF (IR.GT.0) CALL WRGP(AC2,SDM2,VA2,AICC2,AICU2,IP1,
     1      IP2,IS,NC,PAICC2,PAICU2)  
          END IF
          NC=NC+1
          AICC=0.0
          AICU=0.0
 900    CONTINUE
          write(8,*) pAICC2,pAICU2
          WRITE(8,9039)
C*****************************************************************
C      ESTIMATION OF THE FIFTH MODEL: TSAR IN FIRST PIECE
C                                     TAR IN SECOND PIECE
C      TESTING: NO POSSIBLE TESTING IN FIRST PIECE
C               LM  GAPS TESTING IN SECOND PIECE
C*****************************************************************       
       N=N1
       NC=1
       PAICC=0.0
       PAICC2=0.0
       PAICU=0.0
       PAICU2=0.0
         CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
       CALL IZEROVEC(KKARAY,THMAX+1,2)
         KKARAY(2)=IPAZ(2)+I2PAZ(2)*IS
       KKARAY(1)=MAX0(I2PAZ(1),1)*IS
       CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,1,ID,NOBS,IER,IES,is)
C      *** BEGIN LOOP PIECES TO BE ESTIMATED ***
       DO 1100 ICP=1,2
C      *** ASIGNATION OF VALUES ***
          N=NOBS(ICP)
          KR=KKARAY(ICP)
          IP1=IPAZ(ICP)
          IP2=I2PAZ(ICP)
          IF (ICP.EQ.1.AND.IP2.EQ.0) IP2=1
          KRL=IP1+1
          K1=KR+2
          K2=KR+1
          S2=0.0
          I9=0
          IL=IP1+IP2*KRL
C
C       COMPUTATION OF THE SCORE VECTOR AND INFORMATION MATRIX
C
 8000     IF (I9.GT.0) GOTO 8300
C
C       COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE GAPS MODEL
C
            CALL ZEROVEC(D0,LMAX,LMAX)
            CALL ZEROMAT(SAX,NMAX,NMAX,LMAX)
            CALL ZEROMAT(SAXI,NMAX,NMAX,LMAX)
            CALL ZEROMAT(A,LMAX,LMAX,LMAX)
            CALL ZEROVEC(SDM,LMAX,LMAX)
            CALL ZEROVEC(AC1,LMAX,LMAX)
            CALL ZEROVEC(A2,LMAX,LMAX)
            CALL ZEROVEC(C2,LMAX,LMAX)
            CALL ZEROVEC(B0,LMAX,LMAX)
            CALL ZEROVEC(B1,LMAX,LMAX)
            CALL ZEROMAT(B,LMAX,LMAX,LMAX)
            CALL ZEROMAT(B2,LMAX,LMAX,LMAX)
            CALL ZEROMAT(B9,LMAX,LMAX,1)
            CALL ZEROMAT(B11,LMAX,LMAX,1)
            CALL ZEROMAT(Q,LMAX,LMAX,1)
            CALL ZEROVEC(A0,LMAX,LMAX)
            IF (ICP.EQ.1) THEN
               DO 1125 I1=1,N
                    DO 1135 I2=1,IP2+1
 1135              SAX(I1,I2)=AX(I1,(I2-1)*IS+1,ICP)
 1125           SAX(I1,IP2+2)=AX(I1,IS*IP2+2,ICP)
               K1=IP2+2
               K2=IP2+1
             IP1=0
               CALL EQMAT(SAX,SAXI,NMAX,N,K1)
            ELSE IF(ICP.EQ.2) THEN
               DO 1105 I1=1,N
                    DO 1105 I2=1,K1
                     SAX(I1,I2)=AX(I1,I2,ICP)
 1105              SAXI(I1,I2)=SAX(I1,I2)
            END IF
            CALL HUSHLD(SAX,N,K1,NMAX)
            CALL ARMFIT(SAX,NMAX,AC1,K2,N,VA,IMIN,AICC,AICU)
            CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
            IF (ICP.EQ.2.AND.IND.GT.0) THEN
             DO 1131 I1=1,N
                  DO 1141 I3=1,IP2+1
                       DO 1141 I2=1,KRL
 1141              SAG(I1,(I3-1)*KRL+I2)=AX(I1,IS*(I3-1)+I2,ICP)
 1131        SAG(I1,(I3-1)*KRL+1)=AX(I1,K1,ICP)
               CALL HUSHLD(SAG,N,IL+2,NMAX)
               CALL ARMFIT(SAG,NMAX,AC2,IL+1,N,VA2,IMIN,AICC2,AICU2)
               CALL VARCOV(SAG,NMAX,IMIN,VA2,SDM2)
            END IF 
            DO 1110 I1=1,K2
 1110        B1(I1)=AC1(I1)
            S2=VA
            CALL EQVEC(B1,B0,LMAX,K2)
            IF (IND.EQ.0.OR.ICP.EQ.1) THEN
               CALL ZEROVEC(B0,LMAX,K2)         
               DO 1115 I1=1,K2
 1115           A1(I1)=SDM(I1)
                CALL EQVEC(B1,B0,LMAX,K2)
                  CALL ZEROVEC(D0,LMAX,IP1+IS*IP2+1)
               IF (IP1.GT.0) THEN
                    DO 1120 I=1,IP1+1
 1120              D0(I)=B0(I)
               END IF
               I9=1
               GOTO 8200
            END IF
 8300   CONTINUE
          CALL MRQCOF(SAXI,NMAX,S2,N,B1,K2,A,A2,LMAX,FUNCS)
C
C       CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS PARTIAL
C       DERIVATIVES GAP TEST LM
C
 8700   CONTINUE
          IF (ICP.EQ.2) CALL GAP(A2,C2,C5,A0,LMAX,A,Q,B,B2,ELP,B9,
     1        B11,LMAX,IP1,IP2,IS)
          D9=ABS(A0(1))
          DO 1200 I=1,K1
 1200        IF(D9.LE.ABS(A0(I))) D9=ABS(A0(I))
          CALL ZEROVEC(B1,LMAX,K2)
          CALL ZEROVEC(AB0,LMAX,K2)
          CALL ORDER(B0,AB0,LMAX,IP1,IP2,IS,0)
          DO 1205 I=1,K2
 1205        B1(I)=AB0(I)+A0(I)
          S2=S2+A0(K1) 
          IF(D9.LE.(1.D-3).OR.I9.EQ.I0) THEN
             CALL EQVEC(AB0,B0,LMAX,K2)
             GOTO 8800
          END IF
          CALL ZEROVEC(AB1,LMAX,K2)
          CALL ORDER(B1,AB1,LMAX,IP1,IP2,IS,1)
          CALL EQVEC (AB1,B0,LMAX,K2)
          IF (IP1.GT.0) THEN
             DO 1220 I=1,IP1+1
 1220           D0(I)=B0(I)
          END IF
          I9=I9+1
          GOTO 8000
 8800   CALL ZEROVEC(A1,LMAX,K2)
          DO 1230 I=1,IL+1
 1230      A1(I)=SQRT(B(I,I)/N)
          C3=0.0
          DO 1250 I1=IL+1,K2
             DO 1250 I2=IL+1,K2
 1250         C3=C3+B1(I1)*B1(I2)*B(I1,I2)
          C3=N*C3
        AICC=N*LOG(S2)+((2*N*(K2+2))/(N-K2-3))
        AICU=AICC+N*LOG(DBLE(N/REAL(N-K2-2)))
 8200   CONTINUE
          PAICC2=PAICC+AICC2
          PAICC=PAICC+AICC
          PAICU2=AICU+AICU2
          PAICU=PAICU+AICU
          WRITE(8,*) ' '
          WRITE(8,9038)
          IF (ICP.EQ.1) THEN
             CALL WRMULT(B1,A1,B0,D0,I9,IP1,IP2,IS,S2,
     1               AICC,AICU,IR,NOBS(ICP),NC,PAICC,PAICU,C3)
          ELSE IF(ICP.EQ.2) THEN
            CALL WRITEGAP(B1,A1,B0,I9,IP1,IP2,IS,S2,AICC,AICU
     1           ,NOBS(ICP),NC,PAICC,PAICU,C3,IND)
          IF (IR.GT.0) CALL WRGP(AC2,SDM2,VA2,AICC2,AICU2,IP1,
     1      IP2,IS,NC,PAICC2,PAICU2)  
          END IF
          NC=NC+1
          AICC=0.0
          AICU=0.0
 1100   CONTINUE
 1190   continue
C
C       ***  WRITING FORMATS  ***
C
 9000 FORMAT (1X,'TIME SERIES: ',A50)
 9001 FORMAT (1X,'DEGREE OF DIFFERENCING: ',I3)
 9002 FORMAT (1X,'DEGREE OF SEASONAL DIFFERENCING: ',I3)
 9003 FORMAT (1X,'NUMBER OF OBSERVATIONS: ',I6)
 9004 FORMAT (1X,'PERIOD OF SEASONALITY: ',I3)
 9006 FORMAT (1X,'METHOD OF ESTIMATION: SCORING')
 9007 FORMAT (1X,'MEAN',F10.5,5X,'VARIANCE',F10.5)
 9008 FORMAT (1X,'DELAY: ',I3)
 9015 FORMAT (1X,'THRESHOLD: ',F10.5)
 9020 FORMAT (A50)
 9021 FORMAT (A12)
 9024 FORMAT (1X,'PIECE NUMBER: ',I3,3X,'NUMBER OF DATA: ',I6)
 9030 FORMAT (1X,'SELECTION CRITERION: AICC')
 9031 FORMAT (1X,'SELECTION CRITERION: AICU')
 9032 FORMAT (1X,'FIRST MODEL: ONE PIECE, AR')
 9033 FORMAT (1X,'SECOND MODEL: FIRST PIECE, AR, SECOND PIECE, AR')
 9034 FORMAT (1X,'THIRD MODEL: FIRST PIECE:AR*SAR;SECOND PIECE:AR*SAR')
 9037 FORMAT (1X,'FOURTH MODEL: FIRST PIECE: AR*SAR; SECOND PIECE: AR')
 9038 FORMAT (1X,'FIFTH MODEL: FIRST PIECE: AR; SECOND PIECE: AR*SAR')
 9039 FORMAT ('========================================================
     1============',//)
      CLOSE(7)
      CLOSE(9)
      END


      SUBROUTINE NFR(STV,N,IES,THD,NTHD,ID,CMATX,NOBS,
     1    IP1T,IP2T,IS,SIMV,DENORM,ANORM,NMAX)
C===================================================================
C===================================================================
C Purpose: to calculate the normalized fitted residuals
C          and detailed diagnostics.
C Input: STV = array of data;
C        N = its length;
C        IES = starting position from which fitted residuals are
C             calculated;
C        THD = threshold value;
C        NTHD+1 = number of pieces;
C        ID = delay;
C        CMATX(I,J) = Jth coefficient of the Ith AR-model;
C        VV(I) = the sample sum of squared errors for the Ith ar model;
C        NOBS(I) = number of data falling in the Ith piece;
C Output: SIMV(IS+1)-SIMV(N) contains the normalized residuals.
C SUBROUTINES CALLED DESACF,HIST,LINMLD,NORPLT
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION ANORM(*)
      DIMENSION STV(*),SIMV(*),CMATX(2,*),
     1      P(1000),DENORM(*)
      DIMENSION NOBS(*)
      DIMENSION IP1T(*),IP2T(*)
      INTEGER INDEX(1000)
      OPEN(10,FILE='res.out',STATUS='UNKNOWN')
        BNO=0.0
      DO 878 I=1,2
  878 ANORM(I)=0.0
      WRITE(10,100)
  100 FORMAT(//' ORIGINAL DATA',5x,'FITTED DATA',5x,'RESIDUALS')
      DO 2 I=IES,N
C.....Sorting.
         IP=1
         IF (NTHD.EQ.0) GOTO 3
         aaa=0.0
         do 32 j=id,is+id-1
 32         aaa=aaa+stv(i-j)
         IF (aaa.GT.THD) ip=2
  3      CONTINUE
C.....Remember which piece by recording in 'INDEX'
         INDEX(I)=IP
C.....Calculation of fitted residuals:
         NPARM=1+IP1T(IP)+IP2T(IP)*(IP1T(IP)+1)
         T=DBLE(CMATX(IP,1))
         IF (NPARM.LT.2) GOTO 8
C.....Create vectors P: 
         IF (IP1T(IP).GT.0) THEN
            IBO=0
            DO 10 I3=1,IP2T(IP)+1
               DO 10 I2=1,IP1T(IP)
                  IF (I3.GT.1) IBO=1
                  IM=IS*(I3-1)+I2-1*(IBO)
 10           P(1+(I3-1)*(IP1T(IP))+I2)=STV(I-IM)
         ELSE 
            DO 30 I3=1,IP2T(IP)
               IM=IS*I3
 30            P(I3+1)=STV(I-IM)
         END IF
C.....Calculate fitted values:
         DO 5 K=2,NPARM
    5       T=DBLE(CMATX(IP,K))*DBLE(P(K))+T
    8       SIMV(I)=DBLE(STV(I))-T
C.....Writing original data, fitted data and residuals.
            WRITE(10,101) STV(I),T,SIMV(I)
  101       FORMAT(f12.4,6x,f10.4,6x,f10.4)
      
C.....Calculation of sum of square of residuals piece by piece.
C.....ANORM(I) finally stores the residual sum of squares in the Ith piece.
          AN=DBLE(SIMV(I))*DBLE(SIMV(I))
            ANORM(IP)=AN+ANORM(IP)
    2       CONTINUE
            IEND=NTHD+1
            DO 17 I=1,IEND
               FNOB=NOBS(I)
               ANORM(I)=ANORM(I)/DBLE(FNOB)
   17          ANORM(I)=SQRT(ANORM(I))
C.....ANORM(I) stores root mean square of residuals in the Ith piece.
      DO 27 I=IES,N
         IP=INDEX(I)
         II=I-IES+1
C.....Normalize the fitted residuals. These are returned to the
C.....main program via subroutine RAIC, in order to use them in
C.....estimating the uncertainty of the threshold values for a
C.....given model by a bootstrap pocedure. Store the root mean square
C.....used for normalization in the vector DENORM. DENORM is used later
C.....on to denormalize resampled residuals.
      SIMV(II)=DBLE(SIMV(I))/ANORM(IP)
      DENORM(II)=ANORM(IP)
   27 CONTINUE
      RETURN
      END


      SUBROUTINE MULTMAT(AMAT,BMAT,CMAT,ID1MAX,ID2MAX,L1,L2,L3)
C**********************************************************************
C     PORPOSE: MATRIX MULTIPLICATION
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C            BMAT, MATRIX DIMENSION L2,L3
C     OUPUT: CMAT, MATRIX DIMENSION L1,L3
C**********************************************************************


      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID2MAX,*),CMAT(ID1MAX,*)
      EXTERNAL ZEROMAT


      CALL ZEROMAT(CMAT,ID1MAX,L1,L3)
     
C
      DO 99 J=1,L3
         DO 98 K=1,L2
            IF (BMAT(K,J).EQ.0.0) GOTO 98
            DO 97 I=1,L1
               CMAT(I,J)=CMAT(I,J)+AMAT(I,K)*BMAT(K,J)
   97       CONTINUE
   98    CONTINUE
   99 CONTINUE
C
      RETURN
      END


   
      SUBROUTINE MATINV(AMAT,BMAT,ELP,ID1MAX,L1,IFAIL)
C************************************************************************
C     PORPOSE: MATRIX INVERSION
C     INPUT: AMAT, MATRIX DIMENSION L1,L1
C     OUTPUT: BMAT, MATRIX DIMENSION L1,L1
C             IFAIL, MONITORING VALUE
C************************************************************************


      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*),ELP(ID1MAX,*)
      EXTERNAL EQMAT,ZEROMAT
C     
      CALL ZEROMAT(ELP,ID1MAX,L1,L1)
      CALL EQMAT(AMAT,ELP,ID1MAX,L1,L1)
      DO 115 I=1,L1
         BMAT(I,I)=1.0
  115 CONTINUE
      DO 99 J=1,L1
         DO 98 I=J,L1
            IF (ELP(I,J).NE.0.0) GOTO 11
   98    CONTINUE
         IFAIL=1
         RETURN
   11    CONTINUE
      DO 97 K=1,L1
C
         PIVOT01= ELP(J,K)
         ELP(J,K)= ELP(I,K)
         ELP(I,K)= PIVOT01
C
         PIV= BMAT(J,K)
         BMAT(J,K)=BMAT(I,K)
         BMAT(I,K)= PIV
   97 CONTINUE
C
      IF(ELP(J,J).NE.0.0) GOTO 22
C
      IFAIL=1
      RETURN
C
C
   22 CONTINUE
      PPP= 1.0/ELP(J,J)
      DO 96 K=1,L1
C
         ELP(J,K)= ELP(J,K)*PPP
         BMAT(J,K)= BMAT(J,K)*PPP
   96 CONTINUE
C
      DO 95 K=1,L1
C
         IF(J.EQ.K) THEN
C
   33       CONTINUE
C
         ELSE
            PPP= ELP(K,J)
            DO 94 L=1,L1
C
               ELP(K,L)=ELP(K,L)-ELP(J,L)*PPP
               BMAT(K,L)=BMAT(K,L)-BMAT(J,L)*PPP
   94       CONTINUE
C
         END IF
   95 CONTINUE
   99 CONTINUE
C
      RETURN
      END


     
      SUBROUTINE ZEROMAT(AMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: CREATE A MATRIX OF ZEROS
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: AMAT, ZEROED MATRIX DIMENSION L1,L2
C**************************************************************
      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*)
      DO 98 J=1,L2
         DO 99 I=1,L1
            AMAT(I,J)=0.0
  99     CONTINUE
  98  CONTINUE
      RETURN
      END


      SUBROUTINE ZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PORPOSE: CREATE A VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
        DIMENSION VEC(ID1MAX)


      DO 98 I=1,L1     
                VEC(I)=0.0
  98  CONTINUE
      RETURN
        END
   
      SUBROUTINE EQMAT(AMAT,BMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: EQUALIZATION OF MATRICES 
C     INPUT: AMAT, MATRIX DIMENSION L1 L2
C     OUPUT: BMAT, MATRIX EQUAL AMAT DIMENSION L1 L2
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*)
C
      DO 98 J=1,L2
         DO 99 I=1,L1
            BMAT(I,J)=AMAT(I,J)
   99    CONTINUE
   98 CONTINUE
      RETURN
      END


      SUBROUTINE EQVEC(VECA,VECB,ID1MAX,L1)
C******************************************************************
C     PORPOSE: EQUALIZATION OF VECTORS
C     INPUT: VECA, VECTOR DIMENSION L1
C     OUTPUT: VECB, VECTOR EQUAL VECA DIMENSON L1
C******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VECA(ID1MAX),VECB(ID1MAX)
C
      DO 99 I=1,L1
            VECB(I)=VECA(I)
   99 CONTINUE
      RETURN
      END


      SUBROUTINE TRNMAT(AMAT,BMAT,ID1MAX,L1,L2)
C********************************************************************
C     PORPOSE: TRANSPOSE A MATRIX OF DIMENSION L1,L2
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: BMAT, TRANSPOSED MATRIX DIMENSION L2,L1
C********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*)
C
      DO 98 J=1,L2
         DO 99 I=1,L1
            BMAT(J,I)=AMAT(I,J)
   99     CONTINUE
  98  CONTINUE
      RETURN
      END


      SUBROUTINE SORT(Y,N,AX,IAX1,IAX2,KKARAY,THD,NTHD,ID,NOBS,IER,
     1 IES,is)
C***********************************************************************
C     PORPOSE: SORT THE DATA FOR WHICH AN APPROPIATE SAR-MODEL APPLIES
C     INPUT: Y, DATA
C            N, ITS DIMENSION
C            THD, ARRAY CONTAINING THE THRESHOLDS
C            2, NUMBER OF PIECES
C            ID, DELAY PARAMETER
C     OUTPUT:AX, ARRAY CONTAINING THE SORTED DATA
C            IER, ERROR CODE 0 OK
C                            1 SOME PIECES DO NOT HAVE ENOUGH DATA TO
C                                    ESTIMATE THE AR-MODEL
C            IES, START POSITION OF SORTING
C
C            NOBS(I), NUMBER OF DATA IN THE I-TH PIECE
C ***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION NOBS(*),Y(*),THD(*)
      DIMENSION AX(IAX1,IAX2,*),KKARAY(*)
C     *** NUMBER OF PIECES ***
      JEND=NTHD+1
C     *** CLEANING NOBS  ***
      
      DO 10 I=1,JEND
 10      NOBS(I)=0


      
C     ***  CHOSE APPROPIATE STARTING POSITION  ***
      IES=is+ID-1
      DO 20 I1=1,JEND
         KK=KKARAY(I1)
 20      IES=MAX0(IES,KK)
      IES=IES+1
C     *** SORT X INTO AX'IS ***
      DO 30 I=IES,N
         ICP=1
C     *** ASSIGN TO FIRST PIECE FOR NTHD=0 ***
         IF (NTHD.EQ.0) GOTO 50
C     *** FIRST ASSIGN TO LAST PIECE ***
         ICP=NTHD+1
           I1=I-ID      
C     *** SEE Y(IT) BELONGS TO WHICH PIECE
         DO 40 K=1,NTHD
            aaa=0.0
            do 32 j=id,is+id-1
 32            aaa=aaa+y(i-j)
            IF (aaa.GT.THD(K)) GOTO 40
            ICP = K
C     *** AH, IT'IS FOUND ***
            GOTO 50
  40     CONTINUE
  50     CONTINUE
C     *** INDICATE THE CURRENT POSITION X(I) OCCUPIES IN THE ICP PIECE
         NOBS(ICP)=NOBS(ICP)+1
         NOB=NOBS(ICP)
         KEND=KKARAY(ICP)+1
C     *** ASSIGN AX(NOB,I)
         DO 60 K=1,KEND
           AX(NOB,K,ICP)=Y(I-K+1)
  60     CONTINUE
C     *** REASSIGN AX TO AVOID CASE KK=0
         AX(NOB,1,ICP)=1.0
         KEND=KEND+1
         AX(NOB,KEND,ICP)=Y(I)
  30  CONTINUE
C     *** CHECK IF ENOUGH DATA TO ESTIMATE THE MODEL ***
      IER=0
      DO 70 J=1,JEND
         NN=NOBS(J)
         IF (NN.GE.20) GOTO 70
         IER=1
C     *** The JTH REGIME LACKS DATA, SO RETURN WITH IER=1 ***
         RETURN
  70  CONTINUE
      RETURN
      END


      SUBROUTINE COVAR(AX,IIAX1,IIAX2,N,KARAY,IP1,IS,IICP,S,CO,lmax)
C*****************************************************************
C     PORPOSE: CALCULATE THE COVARIANCES NEEDED FOR THE ESTIMATION
C     INPUT: AAX, SORTED DATA AX
C            N, NUMBER OF OBSERVATIONS OF THE PIECE
C            KARAY, ORDER OF THE COVARIANCES
C            IICP, ACTUAL PIECE
C     OUTPUT: S, MEAN
C             CO, ARRAY OF COVARIANCES
C*****************************************************************


      IMPLICIT REAL*8 (A-H,O-Z)
      EXTERNAL ZEROVEC
      DIMENSION AX(IIAX1,IIAX2,*),CO(*),AX1(1000,100,2)


      S=0.0
      K=KARAY+2
      DO 5 IT=1,N
         S=S+AX(IT,K,IICP)
  5   CONTINUE
      S=S/N
      DO 6 I1=2,K
         DO 6 IT=1,N
  6         AX1(IT,I1,IICP)=AX(IT,I1,IICP)-S
      CALL ZEROVEC(CO,LMAX,KARAY+1)
      CO(1)=0.0
      IP2=(KARAY-IP1)/IS
      DO 7 I=2,KARAY+2
         CO(I)=0.0
              DO 8 IT=1,N
         CO(I)=CO(I)+AX1(IT,I,IICP)*AX1(IT,K,IICP)
  8            CONTINUE
      IF (I.EQ.KARAY+2) GOTO 9
           CO(I)=CO(I)/(N-IP1+IP2*IS-I+2)
  7     CONTINUE
  9   CO(1)=CO(I)/(N-IP1+IP2*IS)
      RETURN
      END


      SUBROUTINE PRELIM(B0,D0,C4,lmax,B1,IMX,IP1,IP2,IS)
C*****************************************************************
C       PORPOSE: PROVIDE LNRC PRELIMINAR ESTIMATES
C     INPUT: B1, VECTOR OF ESTIMATES
C     OUTPUT: D0, VECTOR OF ESTIMATES NONMULTIPLICATIVE FORM
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B0(*),D0(*),C4(*)
      DIMENSION B1(*)
      EXTERNAL ZEROVEC,EQVEC
C
      I5=IP1+IP2*(IP1+1)+1
      CALL ZEROVEC(B0,LMAX,IP1+IP2)
      DO 117 I=1,IP1+1
 117     B0(I)=B1(I)
      DO 118 I=1,IP2
 118     B0(IP1+I+1)=B1((I-1)*(IP1+1)+IP1+2)
      CALL ZEROVEC(D0,LMAX,I5)
      IF(IP1.GT.0) THEN
         DO 13 I=1,IP1+1
 13         D0(I)=B0(I)
      END IF
      IF (IP2.GT.0) THEN
           CALL ZEROVEC(C4,LMAX,IP1+IP2+1)
           C4(1)=1.0
           DO 14 I=1,IP1+IP2
 14         C4(I+1)=B0(I+1)
           DO 15 I1=1,IP2
              DO 15 I2=1,IP1+1
 15            D0(IP1+(IP1+1)*(I1-1)+I2+1)=C4(I2)*C4(IP1+1+I1)
      END IF
      CALL EQVEC(D0,B0,IMX,I5)
      CALL ZEROVEC(D0,IMX,IP1+IS*IP2+1)
      IF (IP1.GT.0) THEN
         DO 30 I=1,IP1+1
 30         D0(I)=B0(I)
      END IF
      IF (IP2.GT.0) THEN
         DO 31 I1=1,IP2
            DO 31 I2=1,IP1+1
 31            D0(IS*I1+I2)=B0(IP1+I2+(I1-1)*(IP1+1)+1)
      END IF
      RETURN
        END


      SUBROUTINE GAP(A2,C2,C5,A0,ILX,A,Q,B,B2,ELP,B9,B11,IMX,IP1,
     1               IP2,IS)
C*****************************************************************
C       PORPOSE: CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS 
C              PARTIAL DERIVATIVES GAPS TESTING
C     INPUT: A2, VECTOR OF SCORES
C            A, INFORMATION MATRIX
C            IFS, FIRST ORDER GAP TESTING
C            IR, NUMBER OF ORDERS TO BE TESTED (BEGINS @ IFS)
C     OUTPUT: A0, VECTOR OF ADJUSTING FACTORS
C             B, MATRIX OF RESTRICTED ESTIMATION
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A2(*),C2(*),C5(*),A0(*)
      DIMENSION Q(IMX,*),B(IMX,*),A(IMX,*),B2(IMX,*),ELP(IMX,*)
      DIMENSION B9(IMX,*),B11(IMX,*)
      EXTERNAL ZEROVEC,ZEROMAT,MULTMAT,MATINV,EQMAT


C     INTRODUCTION OF SCORES
      KRP=IP1+IP2*IS
      IR=IS-IP1-1
      IL=IP1+IP2*(IP1+1)
      CALL ZEROVEC(C2,IMX,IL+IR)
      DO 10 I=1,KRP+2
 10      C2(I)=A2(I)
      C5(KRP+2)=C2(KRP+2)
C     PERMUTATION OF SCORE VECTOR
      DO 15 I1=1,IP2+1
         DO 15 I=1,IP1+1
 15         C5(I+(I1-1)*(IP1+1))=C2(I+(I1-1)*IS) 
      DO 20 I1=1,IP2
         DO 20 I=1,IR
 20         C5(1+IL+I+(I1-1)*IR)=C2(1+IP1+I+(I1-1)*IS)
      CALL ZEROMAT(B,IMX,KRP+1,KRP+1)
      CALL ZEROMAT(B2,IMX,KRP+1,KRP+1)
C     PERMUTATION OF THE INFORMATION MATRIX
      DO 50 I1=1,IP2+1
         DO 50 I2=1,IP1+1
            DO 50 I3=1,IP2+1
               DO 50 I4=1,IP1+1
                  B(I2+(I1-1)*(IP1+1),I4+(I3-1)*(IP1+1))=
     1            A(I2+(I1-1)*IS,I4+(I3-1)*IS)
 50   CONTINUE
      DO 60 I1=1,IP2
         DO 60 I2=1,IR
            DO 60 I3=1,IP2
               DO 60 I4=1,IR
                  B(1+IL+I2+(I1-1)*IR,1+IL+I4+(I3-1)*IR)=
     1            A(1+IP1+I2+(I1-1)*IR,1+IP1+I4+(I3-1)*IR)
 60   CONTINUE
      DO 62 I1=1,IP2+1
         DO 62 I2=1,IP1+1
            DO 62 I3=1,IP2
               DO 62 I4=1,IR
                  B(I2+(I1-1)*(IP1+1),1+IL+I4+(i3-1)*ir)=
     1            A(I2+(I1-1)*IS,1+IP1+I4+(I3-1)*Is)
                  B(1+IL+I4+(i3-1)*ir,I2+(I1-1)*(IP1+1))=
     1            A(1+IP1+I4+(I3-1)*is,I2+(I1-1)*IS)              
c      WRITE(8,*) I2+(I1-1)*(IP1+1),1+IL+I4,
c     1      I2+(I1-1)*IS,1+IP1+I4+(I3-1)*IR
c      WRITE(8,*) 1+IL+I4,I2+(I1-1)*(IP1+1),
c     1      1+IP1+I4+(I3-1)*IR,I2+(I1-1)*IS
 62   CONTINUE                       
      DO 170 I1=1,KRP+2
         B(KRP+2,I1)=A(KRP+2,I1)
 170     B(I1,KRP+2)=B(KRP+2,I1)
      CALL ZEROMAT(ELP,IMX,KRP+2,KRP+2)
      CALL MATINV(B,B2,ELP,IMX,KRP+2,IFAIL)
      CALL EQMAT(B2,B,IMX,KRP+2,KRP+2)
      CALL ZEROMAT(B9,IMX,KRP+2,1)
      CALL ZEROMAT(B11,IMX,KRP+2,1)
      DO 190 I=1,KRP+2
 190     B9(I,1)=C5(I)
      CALL MULTMAT(B,B9,B11,IMX,IMX,KRP+2,KRP+2,1)
      CALL ZEROVEC(A0,IMX,KRP+2)
        DO 200 I=1,KRP+2
 200     A0(I)=B11(I,1)
      RETURN
      END
C
      SUBROUTINE MULTEST(A2,D0,C2,A0,ILX,A,Q,B,B2,ELP,B9,B11,IMX,
     1                   IP1,IP2,IS)


C*****************************************************************
C     PORPOSE: CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS 
C              PARTIAL DERIVATIVES MULTIPLICATIVE TESTING
C     INPUT: A2, VECTOR OF SCORES
C            D0, NON MULTIPLICATIVE VECTOR OF ESTIMATES
C            A, INFORMATION MATRIX
C     OUTPUT: A0, VECTOR OF ADJUSTING FACTORS
C             B, MATRIX OF RESTRICTED ESTIMATION
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A2(*),C2(*),D0(*),A0(*)
      DIMENSION Q(IMX,*),B(IMX,*),A(IMX,*),B2(IMX,*),ELP(IMX,*)
      DIMENSION B9(IMX,*),B11(IMX,*)
      EXTERNAL ZEROVEC,ZEROMAT,MULTMAT,MATINV,EQMAT
C
C       CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS PARTIAL
C       DERIVATIVES
C
      I5=IP1+IP2*(IP1+1)+2
      CALL ZEROVEC(C2,IMX,I5+IP1*IP2)
        DO 50 I=1,I5
 50      C2(I)=A2(I)
      IF (IP1.GT.0.AND.IP2.GT.0) THEN
           DO 52 I1=1,IP2
              DO 52 I2=1,IP1
 52         C2(I5+IP1*(I1-1)+I2)=D0(IS*I1+I2+1)-D0(IS*I1+1)*D0(I2+1)
      END IF
        CALL ZEROMAT(Q,IMX,I5,IP1*IP2)
      IF (IP1.GT.0.AND.IP2.GT.0) THEN
        DO 55 I1=1,IP1*IP2
         Q(I5,I1)=0.0
 55      Q(1,I1)=0.0 
           DO 56 I1=1,IP2
              DO 56 I2=1,IP1
                 Q(I2+1,IP1*(I1-1)+I2)=D0(IS*I1+1)
                 Q(1+(IP1+1)*I1,IP1*(I1-1)+I2)=D0(I2+1)
 56          Q(1+(IP1+1)*I1+I2,IP1*(I1-1)+I2)=-1.0
      END IF
      CALL ZEROMAT(B,IMX,I5+IP1*IP2,I5+IP1*IP2)
      CALL ZEROMAT(B2,IMX,I5+IP1*IP2,I5+IP1*IP2)
        DO 60 I1=1,I5
           DO 60 I2=I1,I5
              B(I1,I2)=A(I1,I2)
 60         B(I2,I1)=A(I1,I2)
      DO 64 I1=1,I5
           DO 64 I2=1,IP1*IP2
                B(I1,I5+I2)=Q(I1,I2)
 64         B(I2+I5,I1)=B(I1,I5+I2)
      CALL ZEROMAT(ELP,IMX,I5+IP1*IP2,I5+IP1*IP2)
      CALL MATINV(B,B2,ELP,IMX,I5+IP1*IP2,IFAIL)
      CALL EQMAT(B2,B,IMX,I5+IP1*IP2,I5+IP1*IP2)     
      CALL ZEROMAT(B9,IMX,IMX,1)
      CALL ZEROMAT(B11,IMX,IMX,1)
      DO 70 I=1,I5+IP1*IP2
 70      B9(I,1)=C2(I)
      CALL MULTMAT(B,B9,B11,IMX,IMX,I5+IP1*IP2,I5+IP1*IP2,1)
        CALL ZEROVEC(A0,IMX,I5+IP1*IP2)
        DO 72 I=1,I5+IP1*IP2
 72      A0(I)=B11(I,1)
      RETURN
      END



C     
      SUBROUTINE ZEROMAT3(AMAT,ID1MAX,ID2MAX,L1,L2,L3)
C************************************************************
C
C     PORPOSE: CREATE A 3D MATRIX OF ZEROS   
C     INPUT: AMAT, MATRIX TO BE ZEROED
C            L1,L2,L3, DIMENSIONS
C     OUTPUT: AMAT, ZEROED MATRIX
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,ID2MAX,*)


      DO 97 M=1,L3
         DO 98 J=1,L2
            DO 99 I=1,L1
               AMAT(I,J,M)=0.0
  99        CONTINUE
  98     CONTINUE
  97   CONTINUE
C
      RETURN
      END


      SUBROUTINE IZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PORPOSE: CREATE A INTEGER VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      INTEGER ID1MAX,L1,VEC
        DIMENSION VEC(*)


      DO 98 I=1,L1     
                VEC(I)=0
  98  CONTINUE
      RETURN
        END


      SUBROUTINE DETREND(X,N,TIME)
C**************************************************************
C     PORPOSE: DETREND DATA SET
C     INPUT: X, VECTOR OF DATA
C            N, NUMBER OF OBSERVATIONS
C            TIME, TIME VECTOR
C     OUTPUT: X, DETRENTED VECTOR
***************************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*),TIME(*)      
C
C     *** CALCULATE MEAN VALUE ***
C
      SUMX=0.0
      SUMT=0.0
      DO 1 L=1,N
           SUMX=SUMX+X(L)
           SUMT=SUMT+TIME(L)
1     CONTINUE
      SMEANX=SUMX/N
      SMEANT=SUMT/N
C
C     *** GET SQUARED TIME DEVIATIONS AND CROSS PRODUCTS ***
C
      STT=0.0
      SXT=0.0
      DO 2 L=1,N
         STT=STT+(TIME(L)-SMEANT)**2
         SXT=SXT+(X(L)-SMEANX)*(TIME(L)-SMEANT)
2     CONTINUE
C
C     GET OLS ESTIMATES FOR A AND B:  X(T) = A + B*TIME(T)
C
      B=SXT/STT
      A=SMEANX-(B*SMEANT)
C
C     STORE OLS RESIDUALS IN X
C
      DO 3 L=1,N
         X(L)=X(L)-(A+(B*L))
3     CONTINUE
      RETURN
      END
C     
      SUBROUTINE ORDER(B,C,ID1MAX,IP1,IP2,IS,IA)
C************************************************************
C     PORPOSE: PERMUTATE A VECTOR   
C     INPUT: B, VECTOR TO BE PERMUTATED
C            IO, LENGTH OF THE VECTOR
C            IIFS, FIRST POSITION OF PERMUTATION
C            IIR, ORDERS TO BE PERMUTATED
C     OUTPUT: C, VECTOR PERMUTATED
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B(*),C(*)
C     PERMUTATION IN THE VECTOR
      IR=IS-IP1-1
      IL=IP1+IP2*(IP1+1)
      IF (IA.EQ.0) THEN
         DO 10 I1=1,IP2+1
            DO 10 I=1,IP1+1
 10            C(I+(I1-1)*(IP1+1))=B(I+(I1-1)*IS)
         DO 20 I1=1,IP2
            DO 20 I=1,IR
 20            C(1+IL+I+(I1-1)*IR)=B(1+IP1+I+(I1-1)*IS)
       ELSE IF(IA.EQ.1) THEN
         DO 30 I1=1,IP2+1
            DO 30 I=1,IP1+1
 30          C(I+(I1-1)*IS)=B(I+(I1-1)*(IP1+1))
         DO 40 I1=1,IP2
            DO 40 I=1,IR
 40          C(1+IP1+I+(I1-1)*IS)=B(1+IL+I+(I1-1)*IR)
      END IF
      RETURN
      END


C     
      SUBROUTINE WRITEGAP(B1,A1,B0,II,IP1,IP2,IS,S2,AICC,AICU
     1            ,N,NV,PAICC,PAICU,C3,IND)
C************************************************************
C     PORPOSE: WRITING RESULTS   
C     INPUT: 
C     OUTPUT: WRITING FORMAT
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B1(*),A1(*),B0(*)
      WRITE(8,*) ' '
      WRITE(8,9024) NV,N
      WRITE(8,*) ' '
      WRITE(8,9008)
      IF (IR.EQ.0) II=1
      WRITE(8,9009) II,II-1
      I=0
      KRP=IP1+IS*IP2
      IR=IS-IP1-1
      DO 10 I1=1,IP2+1
           DO 10 I2=1,IP1+1
            I=I+1
          WRITE(8,9010) (I2-1)+(I1-1)*IS,B1(I)
     1     ,A1(I),B1(I)/A1(I),B0(I)
 10   CONTINUE   
      IF (IND.EQ.0) THEN
         DO 20 I=1,KRP+1
 20         WRITE(8,9010) I-1,B1(I),A1(I),B1(I)/A1(I),B0(I)
      END IF
      WRITE(8,*) ' '
      WRITE(8,9014) SQRT(S2)
      WRITE(8,9015) AICC
      WRITE(8,9016) AICU
      IF (IND.EQ.0) GOTO 100
      WRITE(8,*) ' '
      WRITE(8,9011) IP1+1,IP1+IR+1
      WRITE(8,9013)  
      WRITE(8,9017) C3,IR*IP2
 100  IF (NV.EQ.2) THEN
         WRITE(8,9025) PAICC
         WRITE(8,9026) PAICU
         WRITE(8,*) ' '
      END IF


C     *** WRITING FORMATS ***


 9008 FORMAT (1X,'AR COEFFICIENTS')
 9009 FORMAT (1X,'PARAM',9X,'EST(',I2,')',7X,'STD ERR',7X,
     1        'T-RATIO',7X,'EST(',I2,')')
 9010 FORMAT (1X,'F(',I3,')',5X,F10.5,4X,F10.5,4X,F10.5,4X,F10.5)
 9011 FORMAT (1X,'TESTED ORDERS: ',I3,'-',I3)
 9013 FORMAT (1X,'LM TYPE TEST FOR THE GAP HYPOTHESIS')
 9014 FORMAT (1X,'RESIDUAL STD.DEV: ',F20.10)
 9015 FORMAT (1X,'AICC: ',F20.5)
 9016 FORMAT (1X,'AICU: ',F20.5)
 9017 FORMAT (1X,'TEST STATISTIC: ',F15.5,6X,'DF: ',I6)
 9024 FORMAT (1X,'PIECE NUMBER: ',I3,3X,'NUMBER OF DATA: ',I6)
 9025 FORMAT (1X,'POOLED AICC: ',F20.5)
 9026 FORMAT (1X,'POOLED AICU: ',F20.5)
 
      RETURN
      END


      SUBROUTINE wrgp(ac2,sdm2,s2,AICC,AICU,ip1,ip2,is
     1             ,nv,pAICC,pAICU)
C************************************************************
C     PORPOSE: WRITING RESULTS of Regime-Specific AR model when
C              gaps orders are not included in the regression
C     INPUT: 
C     OUTPUT: WRITING FORMAT
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION ac2(*),sdm2(*)
      WRITE(8,*) ' '
      WRITE(8,9008)
      WRITE(8,9009) 1
      I=0
      DO 10 I1=1,IP2+1
         DO 10 I2=1,IP1+1
            I=I+1
 10         WRITE(8,9010) (I2-1)+(I1-1)*IS,
     1   AC2(I),SDM2(I),AC2(I)/SDM2(I)
      WRITE(8,*) ' '
      WRITE(8,9014) SQRT(S2)
      WRITE(8,9015) AICC
      WRITE(8,9016) AICU
 100  IF (NV.EQ.2) THEN
         WRITE(8,9025) PAICC
         WRITE(8,9026) PAICU
         WRITE(8,*) ' '
      END IF


C     *** WRITING FORMATS ***
 9008 FORMAT (1X,'AR COEFFICIENTS WHEN GAPS NOT INCLUDED')
 9009 FORMAT (1X,'PARAM',9X,'EST(',I2,')',7X,'STD ERR',7X,
     1        'T-RATIO')
 9010 FORMAT (1X,'F(',I3,')',5X,F10.5,4X,F10.5,4X,F10.5,4X,F10.5)
 9014 FORMAT (1X,'RESIDUAL STD.DEV: ',F20.10)
 9015 FORMAT (1X,'AICC: ',F20.5)
 9016 FORMAT (1X,'AICU: ',F20.5)
 9017 FORMAT (1X,'TEST STATISTIC: ',F15.5,6X,'DF: ',I6)
 9024 FORMAT (1X,'PIECE NUMBER: ',I3,3X,'NUMBER OF DATA: ',I6)
 9025 FORMAT (1X,'POOLED AICC: ',F20.5)
 9026 FORMAT (1X,'POOLED AICU: ',F20.5)
 
      RETURN
      END


      SUBROUTINE WRMULT(B1,A1,B0,D0,II,IP1,IP2,IS,S2,
     1                  AICC,AICU,IND,N,NV,PAICC,PAICU,C3)
C************************************************************
C     PORPOSE: WRITING RESULTS   
C     INPUT: 
C     OUTPUT: WRITING FORMAT
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B1(*),A1(*),B0(*),D0(*)
C
C      *** WRITING RESULTS ***
C 
      WRITE(8,*) ' '
      WRITE(8,9024) NV,N
      IC=0
      IF (IND.EQ.0) THEN
         WRITE(8,*) ' '      
         WRITE(8,9008)
         WRITE(8,*) ' '
         WRITE(8,9009) II,II
       WRITE(8,9010) IC,B1(1),A1(1),B1(1)/A1(1),D0(1)
         IF (IP1.GT.0) THEN
            DO 10 I=1,IP1
 10      WRITE(8,9010) I,B1(I+1),A1(I+1),B1(I+1)/A1(I+1),D0(I+1)
         END IF
         IF (IP2.GT.0) THEN
            DO 20 I1=1,IP2
               DO 20 I2=1,IP1+1
                  I=IP1+I2+(I1-1)*(IP1+1)
 20   WRITE(8,9011) IS*I1+I2-1,B1(I+1),A1(I+1),B1(I+1)/A1(I+1),
     1              D0(IS*I1+I2)
            END IF
            WRITE(8,9014) SQRT(S2)
            WRITE(8,9015) AICC
            WRITE(8,9016) AICU
            IF (NV.EQ.2) THEN
               WRITE(8,9025) PAICC
               WRITE(8,9026) PAICU
               WRITE(8,*)
            END IF
            GOTO 1000
         END IF
         WRITE(8,*) ' '
         WRITE(8,*) ' '
         WRITE(8,9008)
         WRITE(8,*) ' '
         WRITE(8,9009) II,II-1
         WRITE(8,9010) IC,B1(1),A1(1),B1(1)/A1(1),B0(1)
         IF (IP1.GT.0) THEN
            DO 30 I=1,IP1
 30    WRITE(8,9010) I,B1(I+1),A1(I+1),B1(I+1)/A1(I+1),B0(I+1)
         END IF
         IF (IP2.GT.0) THEN
            DO 40 I=1,IP2
               J=(IP1+1)*I
 40    WRITE(8,9011) I,B1(J+1),A1(J+1),B1(J+1)/A1(J+1),B0(J+1)     
        END IF  
        WRITE(8,9014) SQRT(S2)
        WRITE(8,9015) AICC
        WRITE(8,9016) AICU
        IF (IP1*IP2.EQ.0) GOTO 230
        WRITE(8,*) ' '
        WRITE(8,9013)
        WRITE(8,9017) C3,IP1*IP2
 230    CONTINUE
        IF (NV.EQ.2) THEN
           WRITE(8,9025) PAICC
           WRITE(8,9026) PAICU
           WRITE(8,*)
        END IF
 1000  CONTINUE
        
C
C       ***  WRITING FORMATS  ***
C


 9008 FORMAT (1X,'AR COEFFICIENTS')
 9009 FORMAT (1X,'PARAM',9X,'EST(',I2,')',7X,'STD ERR',7X,
     1        'T-RATIO',7X,'EST(',I2,')')
 9010 FORMAT (1X,'F(',I3,')',5X,F10.5,4X,F10.5,4X,F10.5,4X,F10.5)
 9011 FORMAT (1X,'SF(',I3,')',4X,F10.5,4X,F10.5,4X,F10.5,4X,F10.5)
 9013 FORMAT (1X,'LM TYPE TEST FOR THE SEASONALITY HYPOTHESIS')
 9014 FORMAT (1X,'RESIDUAL STD.DEV: ',F20.10)
 9015 FORMAT (1X,'AICC: ',F20.5)
 9016 FORMAT (1X,'AICU: ',F20.5)
 9017 FORMAT (1X,'TEST STATISTIC: ',F15.5,6X,'DF: ',I6)
 9024 FORMAT (1X,'PIECE NUMBER: ',I3,3X,'NUMBER OF DATA: ',I6)
 9025 FORMAT (1X,'POOLED AICC: ',F20.5)
 9026 FORMAT (1X,'POOLED AICU: ',F20.5)
 9030 FORMAT (1X,'CONSTANT',3X,F10.5,4X,F10.5,4X,F10.5)
 
      RETURN
       END                 


      SUBROUTINE HUSHLD(AH1,N,K,IA11)
C===================================================================
C Purpose: HOUSHOLDER TRANSFORMATION
C Input : AH1 = array to be transformed;
C         N = number of columns;
C         K = number of rows;
C         IA11 = number of columns of AH1 in the calling program.
C Output: AH1 = transformed array.
C==================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*),Z(1000)
      TOL=1.0E-30
      NK=MIN0(N,K)
      DO 100 II=1,NK
      H=0.0
      DO 10 I=II,N
      Z(I) = AH1(I,II)
   10 H=H+Z(I)*Z(I)
      IF(H .GT. TOL) GOTO 40
      G=0.0
      GOTO 100
   40 G=SQRT(H)
      F=AH1(II,II)
      IF(F.GE.0.0) G=-G
      Z(II)=F-G
      H=H-F*G
      IF(II .EQ. K) GOTO 100
      II1=II+1
      DO 90 J=II1,NK
      S=0.0
      DO 20 I=II,N
   20 S=S+Z(I)*AH1(I,J)
      S=S/H
      DO 30 I=II,N
   30 AH1(I,J)=AH1(I,J)-Z(I)*S
   90 CONTINUE
  100 AH1(II,II)=G
      RETURN
      END


      SUBROUTINE ARMFIT(AH1,IA11,AC1,K,N,VA,IMIN,AC3,AC4)
C*******************************************************************
C Purpose: given the lags for different pieces, compute the AICC
C          for the AR-model in the Ith piece.
C Input: AH1 = housholder transformed array (so it is upper triangular);
C        IA11 = number of rows of AH1 in the calling program;
C        K = number of columns;
C        N = number of rows (number of data);
C        ISEL = indicator for information criterion.
C Output: AR(IMIN-1) = the chosen AR-model;
C         IMIN = the number parameters of the AR-model;
C         VA = mean sum of squared errors;
C         AC1 = contains the coefficients of the AR-model.
C Here, AC2 and AC3 are working vectors.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*)
      DIMENSION AC1(*)
      K1=K+1
      K1=MIN0(N,K1)
      OSD=0.0E00
      FN=N
      OSD=OSD+AH1(K1,K1)*AH1(K1,K1)
      AC2=OSD/FN
      AC3=FN*LOG(AC2)+((2*FN*(K+2))/(FN-K-3))
      AC4=AC3+FN*LOG(REAL(FN/(FN-K-2)))
      VA=AC2
      IMIN=K1-1
      IF(IMIN .EQ. 1) GOTO 200
      DO 100 M=IMIN,IMIN
         AC1(M)=AH1(M,K1)/AH1(M,M)
         MM1=M-1
         DO 110 II=1,MM1
            I=M-II
            SUM=DBLE(AH1(I,K1))
            I1=I+1
            DO 120 J=I1,M
  120       SUM=SUM-DBLE(AC1(J))*DBLE(AH1(I,J))
  110    AC1(I)=SUM/AH1(I,I)
  100 CONTINUE
      RETURN
  200 AC1(1)=AH1(1,K1)/AH1(1,1)
      RETURN
      END


      SUBROUTINE VARCOV(SAX,IAX,IMIN,VA,SDM)
C===================================================================
C===================================================================
C PURPOSE: TO PROVIDE THE VARIANCE COVARIANCE MATRIX FOR THE
C          ESTIMATES.
C INPUT: SAX=UPPER TRIANGULAR FORM OF THE DESIGN MATRIX
C        IAX=NO. OF COLUMNS OF SAX IN THE CALLING PROGRAM
C        IMIN: SAX IS OF DIMENSION IMIN BY IMIN
C        VA=SUM OF SQUARED ERRORS
C ACTION: THE SD ESTIMATES AND THE VARIANCE COVARIANCE MATRIX IS PRINTED
C SUBROUTINES CALLED: SINV
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SAX(IAX,*),SDM(*)
      DIMENSION TA(1000)
      EXTERNAL SINV
      IC=0
C     TO COMPRESS THE UPPER TRIANGULAR MATRIX SAX INTO ONE
C     DIMENSIONAL VECTOR TA
      DO 2 I2=1,IMIN
      DO 2 I1=1,I2
      IC=IC+1
    2 TA(IC)=SAX(I1,I2)
C     CALCULATE VA*INVERSE OF THE PRODUCT OF SAX TRANSPOSE AND SAX
      CALL SINV(TA,IMIN,VA)   
C     RETIREVE THE DIAGONAL AND TAKE THE SQUARE ROOT; THEY ARE
C     ESTIMATES OF STANDARD ERROR
      I=1
      IP=1
 11   SDM(I)=SQRT(TA(IP))
      I=I+1
      IP=I+IP
      IF(I .GT. IMIN) GOTO 10
      GOTO 11
  10  RETURN
      END


      SUBROUTINE SINV(A,N,VA)
C===================================================================
C===================================================================
C PURPOSE: TO PROVIDE THE PRODUCT OF THE INVERSE A'*A AND VA
C INPUT: A THE UPPER TRIANGULAR MATRIX STACKED UP ROW-WISE
C        N=DIMENSION OF THE TRIANGULAR MATRIX,SO N*(N-1)=LENGTH OF A
C        VA=MEAN SUM OF SQUARE ERROR
C OUTPUT: A=THE DESIRED PRODUCT AS DESCRIBED IN THE PURPOSE
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1000)
      IPIV=N*(N+1)/2
      IND=IPIV
      DO 6 I=1,N
      DIN=1.D0/DBLE(A(IPIV))
      A(IPIV)=DIN
      MIN=N
      KEND=I-1
      LANF=N-KEND
      IF(KEND) 5,5,2
    2 J=IND
      DO 4 K=1,KEND
      WORK=0.D0
      MIN=MIN-1
      LHOR=IPIV
      LVER=J
      DO 3 L=LANF,MIN
      LVER=LVER+1
      LHOR=LHOR+L
    3 WORK=WORK+DBLE(A(LVER)*A(LHOR))
      A(J)=-WORK*DIN
    4 J=J-MIN
    5 IPIV=IPIV-MIN
    6 IND=IND-1
      DO 8 I=1,N
      IPIV=IPIV+I
      J=IPIV
      DO 8 K=I,N
      WORK=0.D0
      LHOR=J
      DO 7 L=K,N
      LVER=LHOR+K-I
      WORK=WORK+DBLE(A(LHOR)*A(LVER))
    7 LHOR=LHOR+L
      A(J)=WORK*DBLE(VA)
    8 J=J+K
      RETURN
      END
C#####################################################
C Quicksort routine from numerical recipes
C#####################################################
      SUBROUTINE QCKSRT(N,ARR)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (M=7,NSTACK=50,FM=7875.,FA=211.,FC=1663.
     *    ,FMI=1.2698413E-4,maxdat=800000)
      DIMENSION ISTACK(NSTACK)
      DIMENSION ARR(maxdat)
C     DOUBLE PRECISION A
      JSTACK=0
      L=1
      IR=N
      FX=0.
10    IF(IR-L.LT.M)THEN
        DO 13 J=L+1,IR
          A=ARR(J)
          DO 11 I=J-1,1,-1
            IF(ARR(I).LE.A)GO TO 12
            ARR(I+1)=ARR(I)
11        CONTINUE
          I=0
12        ARR(I+1)=A
13      CONTINUE
        IF(JSTACK.EQ.0)RETURN
        IR=ISTACK(JSTACK)
        L=ISTACK(JSTACK-1)
        JSTACK=JSTACK-2
      ELSE
        I=L
        J=IR
        FX=MOD(FX*FA+FC,FM)
        IQ=L+(IR-L+1)*(FX*FMI)
        A=ARR(IQ)
        ARR(IQ)=ARR(L)
20      CONTINUE
21        IF(J.GT.0)THEN
            IF(A.LT.ARR(J))THEN
              J=J-1
              GO TO 21
            ENDIF
          ENDIF
          IF(J.LE.I)THEN
            ARR(I)=A
            GO TO 30
          ENDIF
          ARR(I)=ARR(J)
          I=I+1
22        IF(I.LE.N)THEN
            IF(A.GT.ARR(I))THEN
              I=I+1
              GO TO 22
            ENDIF
          ENDIF
          IF(J.LE.I)THEN
            ARR(J)=A
            I=J
            GO TO 30
          ENDIF
          ARR(J)=ARR(I)
          J=J-1
        GO TO 20
30      JSTACK=JSTACK+2
        IF(JSTACK.GT.NSTACK)PAUSE 'NSTACK must be made larger.'
        IF(IR-I.GE.I-L)THEN
          ISTACK(JSTACK)=IR
          ISTACK(JSTACK-1)=I+1
          IR=I-1
        ELSE
          ISTACK(JSTACK)=I-1
          ISTACK(JSTACK-1)=L
          L=I+1
        ENDIF
      ENDIF
      GO TO 10
      END


      SUBROUTINE PCAND(X,N,CTHD,IPL,IPU,IPR)
C**************************************************************
C DEFINES CANDIDATE THRESHOLD VALUES FOR INFLATION RATES
C THRESHOLD VALUES ARE SEARCHED AMONG THE INTERPERCENTILE RANGE
C @ EQUIWIDTH INTERVALS ( # INTERVALS = # VALUES BETWEEN PERC)
C X = MATRIX OF DATA
C N = # DATA
C CTHD = ARRAY OF CANDIDATE THRESHOLD VALUES
C IPL = USER-SUPPLIED LOWER PERCENTILE
C IPU = USER-SUPPLIED UPPER PERCENTILE
C EXTERNAL SUBROUTINE = QCKSRT
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 IPL,IPU
      DIMENSION X(*),CTHD(*),SARR(1000)
      EXTERNAL QCKSRT
      DO 12 I=1,N
 12      SARR(I)=X(I)
      CALL QCKSRT(N,SARR)
      IP1=NINT(IPL*N)
      AMIN=SARR(IP1)
      IP2=NINT(IPU*N)
      AMAX=SARR(IP2)
      IPR=IP2-IP1
      AI=AMAX-AMIN
      AINC=AI/REAL(IPR)
      CTHD(1)=AMIN
      DO 13 J=2,IPR
 13      CTHD(J)=CTHD(J-1)+AINC
      RETURN
      END


      SUBROUTINE MRQCOF(X,LX,SIG,NDATA,A,MA,ALPHA,BETA,
     1          NALP,FUNCS)
C**************************************************************
C     THIS SUBROUTINE PROVIDES THE SCORES AND THE INFORMATION
C     MATRIX OF EACH REGIME.
C     INPUT: X IS THE MATRIX OF DATA EQUAL TO THE SAX MATRIX (OUTPUT OF
C            THE 'SORT' SUBROUTINE.
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS    
C            SIG IS THE PRELIMINAR ESTIMATOR OF SIGMA SQUARE
C            MA # OF PARAMETERS ( CONSTANT + AR ORDER ) ( WE USE MA+1 TO
C            INCLUDE SIGMA-SQUARE DERIVATIVES )
C     OUTPUT: ALPHA INFORMATION MATRIX
C             BETA VECTOR OF SCORES
C     EXTERNAL FUNCTIONS:
C            FUNCS EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES @ EACH
C             POINT
C**************************************************************


      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(LX,*),ALPHA(NALP,*),XI(100),D2YDA(100,100)
     1         ,A(*),BETA(*),DYDA(100)
      EXTERNAL FUNCS,ZEROVEC,ZEROMAT
      CALL ZEROVEC(BETA,nalp,nalp)
      CALL ZEROMAT(ALPHA,nalp,nalp,nalp)
      CALL ZEROVEC(XI,100,100)
      DO 12 J=1,MA+1
           DO 11 K=1,J
 11         ALPHA(J,K)=0.0
 12      BETA(J)=0.0
      pf3=0.0
      pf4=0.0
      DO 15 I=1,NDATA
         XI(MA)=X(I,MA+1)
         DO 20 IQ=2,MA
 20         XI(MA+1-IQ)=X(I,IQ)
      CALL FUNCS(XI,A,YMOD,DYDA,MA,SIG,D2YDA,100,pf3)
        DO 14 J=1,MA+1
         DO 13 K=1,J
 13         ALPHA(J,K)=ALPHA(J,K)+D2YDA(J,K)
      BETA(J)=BETA(J)+DYDA(J)
 14   CONTINUE
      pf4=pf4+pf3
 15   CONTINUE
      BETA(MA+1)=(-0.5*(NDATA*(SIG-(A(1)**2))-PF4))/(SIG**2)
      DO 19 L=1,MA+1
 19      BETA(L)=BETA(L)/NDATA
      DO 16 J=2,MA+1
           DO 16 K=1,J-1
 16         ALPHA(K,J)=ALPHA(J,K)
      ALPHA(MA+1,MA+1)=(0.5*(NDATA*(SIG-(2*(A(1)**2))-PF4)))/(SIG**3)
      DO 18 J=1,MA+1
         DO 18 K=1,MA+1
 18         ALPHA(J,K)=-ALPHA(J,K)/NDATA
      RETURN
      END


      SUBROUTINE FUNCS(X,A,Y,DYDA,NA,S2,D2YDA,NALP,pf3)
C**********************************************************************
C     THIS SUBROUTINE EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES OF
C     THE LOG-LIKELIHOOD FUNCTION OF AN AR PROCESS 
C     INPUT: X, VECTOR WITH THE VALUE X(T) AND THE REGRESSORS NEEDED 
C            TO FORECAST X(T). ( X(NA)=X(T) X(NA-1)=X(T-1)...X(1)=X(T-P) )
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS
C            S2, SIGMA SQUARE VALUE
C     OUTPUT: DYDA, DERIVATIVE VECTOR @ X(T) 
C             D2YDA, SECOND DERIVATIVES MATRIX @ X(T)
C***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(*),DYDA(*),X(*),D2YDA(NALP,*)
      EXTERNAL ZEROVEC
      Y=0.0
      Q=0.0
      R=0.0
      PA=0.0
      PB=0.0
      PC=0.0
      PD=0.0
      PE=0.0
      PF=0.0
      CALL ZEROVEC(DYDA,100,NA+1)
      DO 1 J=2,NA
         Q=Q+(X(NA-J+1)*A(J))
 1       R=R+((X(NA-J+1)**2)*(A(J)**2))
      PF=X(NA)-(A(1)+Q)
      pf1=x(na)-q
      pf2=pf1-2*a(1)
      pf3=pf1*pf2


C     *** derivative w.r.t constant ***


      DYDA(1)=PF/S2


C     *** derivatives w.r.t. AR parameters ***


      DO 2 K=2,NA
 2        DYDA(K)=(PF*X(NA-K+1))/S2


C 
C     *** derivative w.r.t. sigma-square ***


c      PA=Q*A(1)
c      PB=(X(NA)**2)+(A(1)**2)+R
c      PC=X(NA)*(A(1)+Q)
c      DO 5 L=2,NA
c          DO 5 K=2,L-1
c 5          PD=PD+X(NA-K+1)*X(NA-L+1)*A(J)*A(L)
c      PE=PA+0.5*PB+PD-PC
c      DYDA(NA+1)=((2.0*PE)-S2)/(2.0*(S2**2))


C     *** second derivative w.r.t constant ***


      D2YDA(1,1)=-1.0/S2


C     *** second derivatives w.r.t constant and parameters ***


      DO 3 L=2,NA
 3       D2YDA(L,1)=-X(NA-L+1)/S2


C     *** second derivative w.r.t constant and sigma square ***
 
      D2YDA(NA+1,1)=-PF/(S2**2)


C     *** second derivatives of parameters ***


      DO 4 L=2,NA
         DO 4 K=2,L
 4          D2YDA(L,K)=-(X(NA-L+1)*X(NA-K+1))/S2


C     *** second derivative of sigma-square ***


c      D2YDA(NA+1,NA+1)=(-S2+4.0*PE)/(-2.0*(S2**3))


C     *** second derivative of sigma-square and parameters ***


      DO 6 L=2,NA
 6       D2YDA(NA+1,L)=-(PF*X(NA-L+1))/(S2**2)


      RETURN
      END


      SUBROUTINE LINMUD(X,N)
C===================================================================
C===================================================================
C Purpose: provide the Lin-Mudholkar statistic to test the
C          assumption that the input data X are normally distributed.
C Input: X=data;
C        N=its length.
C Output: the Z-statistic.
C         In the case that the Z-statistic cannot be computed, a 
C         corresponding message is printed.
C Subroutines called: LINMY
C                     BAR,DEM,MMTL
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*),Y(1000)
      EXTERNAL BAR,DEM,MMTL
      DO 1 I=1,N
         T=X(I)
         X(I)=0.0
         CALL LINMY(X,V,N)
         Y(I)=V
    1    X(I)=T
      CALL BAR(X,BX,N)
      CALL BAR(Y,BY,N)
      CALL DEM(X,BX,N)
      CALL DEM(Y,BY,N)
      CALL MMTL(X,Y,N,V1)
      CALL MMTL(X,X,N,V2)
      CALL MMTL(Y,Y,N,V3)
C.....Restore the actual value to X
      CALL DEM(X,-BX,N)
      CALL DEM(Y,-BY,N)
C.....Restore the actual value to X
      R=SQRT(V2*V3)
      R=V1/R
      IF(R .LE. -1.0 .OR. R .GE. 1.0) GOTO 2
      Z=1./2.*LOG((1.+R)/(1.-R))
      Z=Z/SQRT(3./N)
      WRITE(8,100) Z
  100 FORMAT(//' Z-STATISTICS =',E15.6)
      RETURN
    2 CONTINUE
      WRITE(8,200) R,N
  200 FORMAT(//'  R = ',E15.6,' Z CANNOT BE COMPUTED , N = ',I5)
      RETURN
      END


      SUBROUTINE BAR(X,BX,N)
C===================================================================
C===================================================================
C Purpose: provide sample mean of data.
C Input: X = data;
C        N = number of data.
C Output: BX = sample mean.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*)
      DO 1 I=1,N         
    1    WORK=WORK+DBLE(X(I))
      WORK=WORK/DBLE(N)
      BX=WORK
      RETURN
      END


      SUBROUTINE DEM(X,BX,N)
C===================================================================
C===================================================================
C Purpose: mean deletion.
C Input: X = data;
C        N = number of data.
C Output: X = data - sample mean.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*)
      DO 1 I=1,N
    1    X(I)=X(I)-BX
      RETURN
      END


      SUBROUTINE MMTL(X,Y,N,V)
C===================================================================
C===================================================================
C Purpose: to obtain the inner product of X & Y.
C Input: X & Y = arrays of data;
C        N = their common length.
C Output: V = inner product of X & Y.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*),Y(*)
      WORK1=0.0
      DO 1 I=1,N
         T1=X(I)
         T2=Y(I)
         WORK2=DBLE(T1)*DBLE(T2)
    1    WORK1=WORK1+WORK2
      V=WORK1
      RETURN
      END


      SUBROUTINE LINMY(X,V,N)
C===================================================================
C===================================================================
C Purpose: V from X, where V is to be used in calculating
C          the Z-statistic:
C          V = cube root of (Nth of (sum of X squared)-
C            (N-1)th of (square of sum of X))
C Input: X = array of data;
C        N = length of X;
C        V: see discussion above.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*)
        EXTERNAL BAR
      CALL BAR (X,BX,N)
      CALL MMTL(X,X,N,R)
      V=R-BX*BX/(N-1)*N*N
      V=V/N
      ILS=1
      IF(V .LT. 0.0) ILS=-1
      V=ILS*V
      IF(V .EQ. 0.0) GOTO 1
      V=V**0.333333
      V=ILS*V
      RETURN
    1 V=0.0
      RETURN
      END
